@@ -41,6 +41,7 @@ SKIP_PATTERN='(backupfile|bison|copy-file|datetime|exception|exclude|gettext|lin
 # linkat: https://builds.sr.ht/query/log/1402429/test32/log
 # mkdir: https://builds.sr.ht/~alip/job/1286254
 # openat-safer: https://builds.sr.ht/~alip/job/1522333
+# openat2: https://builds.sr.ht/query/log/1604803/test32/log
 # posix_openpt: https://builds.sr.ht/~alip/job/126488
 # readdir: no known fail but best to check everytime.
 # rename: https://builds.sr.ht/~alip/job/1286933
@@ -52,6 +53,7 @@ FAIL_HISTORY=(
     linkat
     mkdir
     openat-safer
+    openat2
     posix_openpt
     readdir
     rename
@@ -88,24 +90,9 @@ edo() {
     "$@"
 }
 
-run_test() {
-    local name="$1"
-    local tdir="$(readlink -f "${DIR}")"
-    local tnam="$(echo "${name}" | sed -e 's|/|-|g')"
+run_tests() {
     [[ -n "${SYD_TEST_DMESG}" ]] && sudo dmesg -C
 
-    echo >&2 "[*] Creating test directory..."
-    ./gnulib-tool \
-        --avoid=pt_chown \
-        --create-testdir "${name}" \
-        --dir "${tdir}/${tnam}" || return $?
-
-    pushd "${tdir}/${tnam}" &>/dev/null
-
-    echo >&2 "[*] Running configure for test ${name}..."
-    ./configure >/dev/null 2>&1 || return $?
-    make -j >/dev/null 2>&1 || return $?
-
     edo \
         timeout -sKILL ${SYD_TEST_TIMEOUT} \
         ${SYD} -ppaludis -m 'allow/all+/***' -mlock:on \
@@ -113,8 +100,7 @@ run_test() {
     r=$?
 
     if [[ $r == 0 ]]; then
-        popd &>/dev/null
-        rm -fr "${tdir}/${tnam}"
+        rm -fr "${DIR}"
         return 0
     fi
 
@@ -128,15 +114,7 @@ run_test() {
         echo '-->8-- KERNEL LOG END   --8<--'
     fi
 
-    echo >&2 "[*] Keeping test directory of failed test ${name}: ${tdir}/${tnam}"
-    #echo >&2 "[*] Rerunning failed test ${name} under strace..."
-    #edo \
-    #    timeout -sKILL ${SYD_TEST_TIMEOUT} \
-    #    strace -f -s256 \
-    #    "${SYD}" -ppaludis -m 'allow/all+/***' -mlock:on \
-    #    -- make -j check >strace.log 2>&1
-
-    popd &>/dev/null
+    echo >&2 "[*] Keeping test directory of failed tests: ${DIR}"
     return $r
 }
 
@@ -169,24 +147,9 @@ fi
 TESTS=( "${FAIL_HISTORY[@]}" "${TESTS[@]:0:${NTEST}}" )
 NTEST=${#TESTS[@]}
 
-idx=0
-for name in "${TESTS[@]}"; do
-    : $(( idx++ ))
-    echo >&2 -e "\033[92m*** $name ($idx of $NTEST: $PASS ok, $FAIL notok, $SKIP todo) ***\033[0m"
-    if echo "${name}" | grep -qE "${SKIP_PATTERN}"; then
-        echo "ok ${idx} - ${name} # TODO"
-        : $(( SKIP++ ))
-    elif run_test "${name}"; then
-        echo "ok ${idx} - ${name}"
-        : $(( PASS++ ))
-    else
-        echo "not ok ${idx} - ${name} - FAIL: $?"
-        : $(( FAIL++ ))
-        [[ -z "${SYD_TEST_QUICK}" ]] || break
-    fi
-done
-
-echo "# $PASS tests passed."
-echo "# $FAIL tests failed."
-echo "# $SKIP tests skipped."
-exit $FAIL
+set -ex
+./gnulib-tool --avoid=pt_chown --create-testdir --dir "${DIR}"/tmp "${TESTS[@]}"
+pushd "${DIR}"/tmp
+./configure
+make -j
+run_tests
