@@ -115,10 +115,10 @@ fn test_libsyd_go() {
         .arg("-plib")
         .arg("-pP") // Allow non-PIE.
         .arg("-eLD_LIBRARY_PATH=") // pass-through
-        .arg("-mtrace/allow_unsafe_libc:1") // or else AT_SECURE prevents pass-through
+        .arg("-mtrace/allow_unsafe_exec_libc:1") // or else AT_SECURE prevents pass-through
         .arg("-mlock:off") // Due to the way go test works, we need this.
         .arg("--")
-        .args(["go", "test", "-ldflags", &format!("-extldflags '-L{libdir}'"), "-v", "-x", "-p", "1"])
+        .args(["go", "test", "-ldflags", &format!("-extldflags '-L{libdir}'"), "-p", "1"])
         .status()
         .expect("execute go test");
     assert!(status.success(), "status:{status:?}");
@@ -139,7 +139,7 @@ fn test_libsyd_pl() {
         .arg("-plib")
         .arg("-pP") // Allow non-PIE.
         .arg("-eLD_LIBRARY_PATH=") // pass-through
-        .arg("-mtrace/allow_unsafe_libc:1") // or else AT_SECURE prevents pass-through
+        .arg("-mtrace/allow_unsafe_exec_libc:1") // or else AT_SECURE prevents pass-through
         .arg("--")
         .arg("./src/syd.pl")
         .status()
@@ -154,7 +154,7 @@ fn test_libsyd_py() {
         .arg("-plib")
         .arg("-pP") // Allow non-PIE.
         .arg("-eLD_LIBRARY_PATH=") // pass-through
-        .arg("-mtrace/allow_unsafe_libc:1") // or else AT_SECURE prevents pass-through
+        .arg("-mtrace/allow_unsafe_exec_libc:1") // or else AT_SECURE prevents pass-through
         .arg("--")
         .arg("./src/syd.py")
         .status()
@@ -179,7 +179,7 @@ fn test_libsyd_rb() {
         .arg("-plib")
         .arg("-pMP") // Allow W^X memory and non-PIE.
         .arg("-eLD_LIBRARY_PATH=") // pass-through
-        .arg("-mtrace/allow_unsafe_libc:1") // or else AT_SECURE prevents pass-through
+        .arg("-mtrace/allow_unsafe_exec_libc:1") // or else AT_SECURE prevents pass-through
         .arg("--")
         .arg("./src/syd.rb")
         .arg("-s0")
