@@ -91,7 +91,7 @@ pub(crate) const fn cmsg_align_32(len: usize) -> usize {
 ///
 /// The total space in bytes required to store the control message, ensuring proper alignment.
 pub(crate) const fn cmsg_space_32(length: u32) -> usize {
-    cmsg_align_32((length as usize).saturating_add(cmsg_align_32(std::mem::size_of::<cmsghdr32>())))
+    cmsg_align_32((length as usize).saturating_add(cmsg_align_32(size_of::<cmsghdr32>())))
 }
 
 /// Computes the byte length of a control message's header and data for a 32-bit system.
@@ -107,7 +107,7 @@ pub(crate) const fn cmsg_space_32(length: u32) -> usize {
 ///
 /// The combined length of the control message header and the data, aligned as required.
 pub(crate) const fn cmsg_len_32(length: u32) -> usize {
-    cmsg_align_32(std::mem::size_of::<cmsghdr32>()).saturating_add(length as usize)
+    cmsg_align_32(size_of::<cmsghdr32>()).saturating_add(length as usize)
 }
 
 /// This structure represents the Linux 32-bit data structure 'struct stat'
@@ -455,7 +455,7 @@ pub unsafe fn getxattrat<Fd: AsFd, P: ?Sized + NixPath>(
                     flags.bits(),
                     name,
                     args as *mut XattrArgs,
-                    std::mem::size_of::<XattrArgs>(),
+                    size_of::<XattrArgs>(),
                 )
             },
         )
@@ -503,7 +503,7 @@ pub unsafe fn setxattrat<Fd: AsFd, P: ?Sized + NixPath>(
                 flags.bits(),
                 name,
                 args as *const XattrArgs,
-                std::mem::size_of::<XattrArgs>(),
+                size_of::<XattrArgs>(),
             )
         })
         .map(drop)
@@ -759,7 +759,7 @@ pub struct DirIter {
     offset: usize,
 }
 
-const DIRENT_ALIGN: usize = std::mem::align_of::<libc::dirent64>();
+const DIRENT_ALIGN: usize = align_of::<libc::dirent64>();
 
 impl DirIter {
     /// Creates a new `DirIter` with an allocated buffer of `bufsiz` bytes.
@@ -1577,7 +1577,7 @@ pub fn epoll_get_params<Fd: AsFd>(fd: Fd) -> Result<EpollParams, Errno> {
 /// Uses getsockopt SO_DOMAIN to get the domain of the given socket.
 pub fn getsockdomain<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
     #[expect(clippy::cast_possible_truncation)]
-    let mut len = std::mem::size_of::<libc::c_int>() as libc::socklen_t;
+    let mut len = size_of::<libc::c_int>() as libc::socklen_t;
     let mut fml: libc::c_int = 0;
 
     // SAFETY: In libc we trust.
