@@ -445,7 +445,7 @@ pub fn confine_scmp_write(
         }
     };
 
-    if cfg!(feature = "prof") && (chk_mem && Sandbox::memory_access() < 2) {
+    if cfg!(feature = "prof") || (chk_mem && Sandbox::memory_access() < 2) {
         ctx.add_rule(ScmpAction::Allow, syscall)?;
         return Ok(());
     }
