@@ -486,7 +486,7 @@ pub fn setsockopt_serial<Fd: AsFd>(fd: Fd, id: KeySerial) -> Result<(), Errno> {
             SOL_ALG,
             ALG_SET_KEY_BY_KEY_SERIAL,
             &raw const id as *const libc::c_void,
-            std::mem::size_of::<KeySerial>() as libc::socklen_t,
+            size_of::<KeySerial>() as libc::socklen_t,
         )
     })
     .map(drop)
