@@ -6,30 +6,32 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::os::fd::{AsFd, AsRawFd};
+use std::{
+    os::fd::{AsFd, AsRawFd},
+    sync::LazyLock,
+};
 
 use libseccomp::{ScmpNotifResp, ScmpSyscall};
 use nix::{
     errno::Errno,
-    fcntl::{AtFlags, OFlag},
+    fcntl::AtFlags,
     sys::stat::{fchmod, Mode},
     NixPath,
 };
-use once_cell::sync::Lazy;
 
 use crate::{
     config::PROC_FD,
-    fs::{safe_open_path, FsFlags},
-    hook::{PathArgs, SysArg, UNotifyEventRequest},
+    fs::FsFlags,
     kernel::{syscall_path_handler, to_atflags, to_mode},
     path::XPathBuf,
+    req::{PathArgs, SysArg, UNotifyEventRequest},
     sandbox::SandboxGuard,
 };
 
 // Note fchmodat2 may not be available,
 // and libc::SYS_fchmodat2 may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_FCHMODAT2: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_FCHMODAT2: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("fchmodat2")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -39,21 +41,20 @@ static SYS_FCHMODAT2: Lazy<libc::c_long> = Lazy::new(|| {
 pub(crate) fn sys_fchmod(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: Reject undefined/invalid mode.
-    let mode = match to_mode(req.data.args[1]) {
-        Ok(mode) => mode,
-        Err(errno) => return request.fail_syscall(errno),
-    };
+    // SAFETY: Strip undefined/invalid mode bits.
+    let mode = to_mode(req.data.args[1]);
 
+    // WANT_READ: fchmod(2) does not work with O_PATH fds.
     let argv = &[SysArg {
         dirfd: Some(0),
+        fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
         ..Default::default()
     }];
     syscall_path_handler(request, "fchmod", argv, |path_args, request, sandbox| {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. SysArg.path is None asserting dir is Some.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
         // SAFETY: We apply force_umask to chmod modes to ensure consistency.
@@ -68,15 +69,11 @@ pub(crate) fn sys_fchmod(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_chmod(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: Reject undefined/invalid mode.
-    let mode = match to_mode(req.data.args[1]) {
-        Ok(mode) => mode,
-        Err(errno) => return request.fail_syscall(errno),
-    };
+    // SAFETY: Strip undefined/invalid mode bits.
+    let mode = to_mode(req.data.args[1]);
 
     let argv = &[SysArg {
         path: Some(0),
-        fsflags: FsFlags::MUST_PATH,
         ..Default::default()
     }];
 
@@ -88,17 +85,13 @@ pub(crate) fn sys_chmod(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_fchmodat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: Reject undefined/invalid mode.
-    let mode = match to_mode(req.data.args[2]) {
-        Ok(mode) => mode,
-        Err(errno) => return request.fail_syscall(errno),
-    };
+    // SAFETY: Strip undefined/invalid mode bits.
+    let mode = to_mode(req.data.args[2]);
 
     // Note: Unlike fchmodat2, fchmodat always resolves symbolic links.
     let argv = &[SysArg {
         dirfd: Some(0),
         path: Some(1),
-        fsflags: FsFlags::MUST_PATH,
         ..Default::default()
     }];
 
@@ -116,11 +109,8 @@ pub(crate) fn sys_fchmodat2(request: UNotifyEventRequest) -> ScmpNotifResp {
         Err(errno) => return request.fail_syscall(errno),
     };
 
-    // SAFETY: Reject undefined/invalid mode.
-    let mode = match to_mode(req.data.args[2]) {
-        Ok(mode) => mode,
-        Err(errno) => return request.fail_syscall(errno),
-    };
+    // SAFETY: Strip undefined/invalid mode bits.
+    let mode = to_mode(req.data.args[2]);
 
     let mut fsflags = FsFlags::MUST_PATH;
     if flags.contains(AtFlags::AT_SYMLINK_NOFOLLOW) {
@@ -147,77 +137,49 @@ fn syscall_chmod_handler(
     mut mode: Mode,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
+    // We use MUST_PATH, dir refers to the file.
+    assert!(
+        path.base.is_empty(),
+        "BUG: MUST_PATH returned a directory for chmod, report a bug!"
+    );
+    let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
+
     // SAFETY: We apply force_umask to chmod modes to ensure consistency.
     let umask = sandbox.umask.unwrap_or(Mode::empty());
     mode &= !umask;
 
-    let fd = path
-        .dir
-        .as_ref()
-        .map(|fd| fd.as_raw_fd())
-        .ok_or(Errno::EBADF)?;
-
-    let flags = if path.base.is_empty() {
-        // FD-only call, e.g remote-fd transfer due to fchmod(2).
-        libc::AT_EMPTY_PATH
+    match if *SYS_FCHMODAT2 > 0 {
+        // SAFETY: No libc wrapper for fchmodat2 yet.
+        Errno::result(unsafe {
+            libc::syscall(
+                *SYS_FCHMODAT2,
+                fd.as_raw_fd(),
+                c"".as_ptr(),
+                mode.bits(),
+                libc::AT_EMPTY_PATH,
+            )
+        })
     } else {
-        // SAFETY: Do not resolve symlinks in base to prevent TOCTTOU.
-        libc::AT_SYMLINK_NOFOLLOW
-    };
-
-    path.base
-        .with_nix_path(|cstr| {
-            match if *SYS_FCHMODAT2 > 0 {
-                // SAFETY: No libc wrapper for fchmodat2 yet.
+        Err(Errno::ENOSYS)
+    } {
+        Ok(_) => Ok(()),
+        Err(Errno::ENOSYS) => {
+            // Fallback to `/proc` indirection,
+            //
+            // path to fd is open already!
+            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
+            pfd.with_nix_path(|cstr| {
+                // SAFETY: We deliberately bypass the libc wrapper here.
                 Errno::result(unsafe {
-                    libc::syscall(
-                        *SYS_FCHMODAT2,
-                        fd.as_raw_fd(),
-                        cstr.as_ptr(),
-                        mode.bits(),
-                        flags,
-                    )
+                    libc::syscall(libc::SYS_fchmodat, PROC_FD(), cstr.as_ptr(), mode.bits())
                 })
-            } else {
-                Err(Errno::ENOSYS)
-            } {
-                Ok(_) => Ok(()),
-                Err(Errno::ENOSYS) if path.base.is_empty() => {
-                    // Fallback to `/proc` indirection,
-                    //
-                    // path to fd is open already!
-                    let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
-                    pfd.with_nix_path(|cstr| {
-                        // SAFETY: We deliberately bypass the libc wrapper here.
-                        Errno::result(unsafe {
-                            libc::syscall(libc::SYS_fchmodat, PROC_FD(), cstr.as_ptr(), mode.bits())
-                        })
-                    })?
-                    .map(drop)
-                }
-                Err(Errno::ENOSYS) => {
-                    // Fallback to `/proc` indirection.
-                    //
-                    // open an `O_PATH` fd without following symlinks.
-                    let fd = safe_open_path(
-                        path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?,
-                        path.base,
-                        OFlag::O_NOFOLLOW,
-                    )?;
-                    let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
-                    pfd.with_nix_path(|cstr| {
-                        // SAFETY: We deliberately bypass the libc wrapper here.
-                        Errno::result(unsafe {
-                            libc::syscall(libc::SYS_fchmodat, PROC_FD(), cstr.as_ptr(), mode.bits())
-                        })
-                    })?
-                    .map(drop)
-                }
-                Err(errno) => Err(errno),
-            }
-        })?
-        .map(|_| request.return_syscall(0))
+            })?
+            .map(drop)
+        }
+        Err(errno) => Err(errno),
+    }
+    .map(|_| request.return_syscall(0))
 }
