@@ -18,9 +18,9 @@ use crate::{
     config::PROC_FILE,
     confine::scmp_arch_bits,
     fs::{file_type, FsFlags},
-    hook::{SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
     path::XPathBuf,
+    req::{SysArg, UNotifyEventRequest},
 };
 
 #[expect(clippy::arithmetic_side_effects)]
