@@ -15,8 +15,8 @@ use crate::{
     compat::getdents64,
     config::{DIRENT_BUF_SIZE, MMAP_MIN_ADDR},
     fs::CanonicalPath,
-    hook::UNotifyEventRequest,
     kernel::sandbox_path,
+    req::UNotifyEventRequest,
     sandbox::Capability,
 };
 
@@ -68,12 +68,24 @@ pub(crate) fn sys_getdents64(request: UNotifyEventRequest) -> ScmpNotifResp {
         let count = usize::try_from(req.data.args[2])
             .or(Err(Errno::EINVAL))?
             .min(DIRENT_BUF_SIZE);
+        if count == 0 {
+            // Reject zero size _after_ ENOTDIR check.
+            return Err(Errno::EINVAL);
+        }
+        // There's no guarantee on the order of items returned by getdents64(2).
+        // Therefore we must potentially check each element for dot or dotdot,
+        // until we actually see one.
+        let mut seen_dot = false;
+        let mut seen_dotdot = false;
         let pid = req.pid();
         let len = dir.len();
-        let mut dot: u8 = 0;
-        let mut ret: u64 = 0;
+        let mut ret: usize = 0;
         while ret == 0 {
-            let mut entries = match getdents64(&fd, count) {
+            let siz = count.checked_sub(ret).ok_or(Errno::EOVERFLOW)?;
+            if siz == 0 {
+                break;
+            }
+            let mut entries = match getdents64(&fd, siz) {
                 Ok(entries) => entries,
                 Err(Errno::ECANCELED) => break, // EOF or empty directory
                 Err(errno) => return Err(errno),
@@ -84,15 +96,16 @@ pub(crate) fn sys_getdents64(request: UNotifyEventRequest) -> ScmpNotifResp {
             let safe_name = !sandbox.flags.allow_unsafe_filename();
             let restrict_mkbdev = !sandbox.flags.allow_unsafe_mkbdev();
 
-            #[expect(clippy::arithmetic_side_effects)]
             for entry in &mut entries {
-                if dot < 2 && entry.is_dot() {
-                    // SAFETY: Allow the special dot entries `.` and `..`.
-                    // Note, `..` may point to a denylisted directory,
-                    // however at this point there's not much we can do:
-                    // even the root directory, ie `/`, has a `..`. In
-                    // this exceptional case `..` points to `.`.
-                    dot += 1;
+                // SAFETY: Allow the special dot entries `.` and `..`.
+                // Note, `..` may point to a denylisted directory,
+                // however at this point there's not much we can do:
+                // even the root directory, ie `/`, has a `..`. In
+                // this exceptional case `..` points to `.`.
+                if !seen_dot && entry.is_dot() {
+                    seen_dot = true;
+                } else if !seen_dotdot && entry.is_dotdot() {
+                    seen_dotdot = true;
                 } else {
                     // Append entry name to the directory.
                     dir.push(entry.name_bytes());
@@ -108,13 +121,13 @@ pub(crate) fn sys_getdents64(request: UNotifyEventRequest) -> ScmpNotifResp {
                         )
                         .is_err()
                     {
-                        // skip entry.
+                        // Skip entry.
                         dir.truncate(len);
                         continue;
                     }
 
-                    // SAFETY: Run sandbox access check with stat capability.
-                    let err = sandbox_path(
+                    // SAFETY: Run sandbox access check with Stat capability.
+                    let hide = sandbox_path(
                         Some(&request),
                         &sandbox,
                         request.scmpreq.pid(), // Unused when request.is_some()
@@ -124,18 +137,28 @@ pub(crate) fn sys_getdents64(request: UNotifyEventRequest) -> ScmpNotifResp {
                         "getdents64",
                     )
                     .is_err();
+
+                    // Restore directory entry.
                     dir.truncate(len);
-                    if err {
-                        // skip entry.
+
+                    if hide {
+                        // Skip entry.
                         continue;
                     }
                 }
 
                 // Access granted, write entry to sandbox process memory.
-                match request.write_mem(entry.as_bytes(), req.data.args[1] + ret) {
+                // Handle truncation as necessary.
+                let buf = entry.as_bytes();
+                let siz = count.checked_sub(ret).ok_or(Errno::EOVERFLOW)?;
+                let siz = buf.len().min(siz);
+                let ptr = req.data.args[1]
+                    .checked_add(ret as u64)
+                    .ok_or(Errno::EOVERFLOW)?;
+                match request.write_mem(&buf[..siz], ptr) {
                     Ok(n) => {
-                        ret += n as u64;
-                        if n != entry.size() {
+                        ret = ret.checked_add(n).ok_or(Errno::EOVERFLOW)?;
+                        if n != entry.size() || ret >= count {
                             break;
                         }
                     }
