@@ -27,7 +27,7 @@ use crate::{
     fs::{fd_status_flags, safe_open_msym, CanonicalPath},
     kernel::sandbox_path,
     path::XPathBuf,
-    proc::{proc_mem, proc_statm},
+    proc::{proc_mem, proc_stat, proc_statm},
     req::UNotifyEventRequest,
     sandbox::{Action, Capability, IntegrityError},
     warn,
@@ -84,31 +84,48 @@ impl Serialize for MemSyscall {
 }
 
 pub(crate) fn sys_brk(request: UNotifyEventRequest) -> ScmpNotifResp {
-    syscall_mem_handler(request, MemSyscall::Brk)
+    let req = request.scmpreq;
+    let size = match proc_stat(req.pid()) {
+        Ok(stat) => req.data.args[0].saturating_sub(stat.startbrk),
+        Err(errno) => return request.fail_syscall(errno),
+    };
+    if size == 0 {
+        // SAFETY: System call wants to shrink memory.
+        // No pointer dereference in size check.
+        return unsafe { request.continue_syscall() };
+    }
+    syscall_mem_handler(request, MemSyscall::Brk, size)
 }
 
 pub(crate) fn sys_mmap(request: UNotifyEventRequest) -> ScmpNotifResp {
-    syscall_mem_handler(request, MemSyscall::Mmap)
+    let req = request.scmpreq;
+    syscall_mem_handler(request, MemSyscall::Mmap, req.data.args[1])
 }
 
 pub(crate) fn sys_mmap2(request: UNotifyEventRequest) -> ScmpNotifResp {
-    syscall_mem_handler(request, MemSyscall::Mmap2)
+    let req = request.scmpreq;
+    syscall_mem_handler(request, MemSyscall::Mmap2, req.data.args[1])
 }
 
 pub(crate) fn sys_mremap(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     let old_size = req.data.args[1];
     let new_size = req.data.args[2];
-    if new_size < old_size {
+    let size = new_size.saturating_sub(old_size);
+    if size == 0 {
         // SAFETY: System call wants to shrink memory.
         // No pointer dereference in size check.
         return unsafe { request.continue_syscall() };
     }
-    syscall_mem_handler(request, MemSyscall::Mremap)
+    syscall_mem_handler(request, MemSyscall::Mremap, size)
 }
 
 #[expect(clippy::cognitive_complexity)]
-fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> ScmpNotifResp {
+fn syscall_mem_handler(
+    request: UNotifyEventRequest,
+    syscall: MemSyscall,
+    size: u64,
+) -> ScmpNotifResp {
     let req = request.scmpreq;
     let caps = syscall.caps();
 
@@ -426,17 +443,17 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
             Ok(statm) => statm.size.saturating_mul(*PAGE_SIZE),
             Err(errno) => return request.fail_syscall(errno),
         };
-        if mem_vm_cur >= mem_vm_max {
+        if mem_vm_cur.saturating_add(size) >= mem_vm_max {
             if mem_act != Action::Filter {
                 if verbose {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
                         "sys": &name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
-                        "tip": "increase `mem/vm_max'",
+                        "mem_size": size, "tip": "increase `mem/vm_max'",
                         "req": &request);
                 } else {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
                         "sys": &name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
-                        "tip": "increase `mem/vm_max'",
+                        "mem_size": size, "tip": "increase `mem/vm_max'",
                         "pid": request.scmpreq.pid);
                 }
             }
@@ -461,17 +478,17 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
             Ok(mem_cur) => mem_cur,
             Err(errno) => return request.fail_syscall(errno),
         };
-        if mem_cur >= mem_max {
+        if mem_cur.saturating_add(size) >= mem_max {
             if mem_act != Action::Filter {
                 if verbose {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
                         "sys": &name, "mem_max": mem_max, "mem_cur": mem_cur,
-                        "tip": "increase `mem/max'",
+                        "mem_size": size, "tip": "increase `mem/max'",
                         "req": &request);
                 } else {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
                         "sys": &name, "mem_max": mem_max, "mem_cur": mem_cur,
-                        "tip": "increase `mem/max'",
+                        "mem_size": size, "tip": "increase `mem/max'",
                         "pid": request.scmpreq.pid);
                 }
             }
