@@ -623,10 +623,7 @@ pub(crate) fn to_mode(arg: u64) -> Mode {
 // to_mode that rejects unknown/invalid bits.
 #[inline]
 pub(crate) fn to_mode2(arg: u64) -> Result<Mode, Errno> {
-    let mode = arg
-        .try_into()
-        .map(|mode: libc::mode_t| mode & !SFlag::S_IFMT.bits())
-        .or(Err(Errno::EINVAL))?;
+    let mode = arg.try_into().or(Err(Errno::EINVAL))?;
     Mode::from_bits(mode).ok_or(Errno::EINVAL)
 }
 
