@@ -12,7 +12,8 @@ use nix::{errno::Errno, sys::socket::SockaddrLike};
 
 use crate::{
     compat::{fstatx, STATX_INO},
-    hook::UNotifyEventRequest,
+    config::MMAP_MIN_ADDR,
+    req::UNotifyEventRequest,
 };
 
 pub(crate) fn handle_getsockname(
@@ -25,7 +26,8 @@ pub(crate) fn handle_getsockname(
     drop(fd); // Close our copy of the socket.
 
     // Lookup path by inode in sandbox unix map.
-    let addr = if let Some((_, Some(addr))) = request.get_unix(inode) {
+    // peer=None check filters out connect(2).
+    let addr = if let Some(addr) = request.get_unix(inode).and_then(|unix| unix.addr) {
         addr
     } else {
         // Not a UNIX domain socket, continue.
@@ -35,7 +37,7 @@ pub(crate) fn handle_getsockname(
 
     // Determine address length.
     let addrlen = if args[2] != 0 {
-        const SIZEOF_SOCKLEN_T: usize = std::mem::size_of::<libc::socklen_t>();
+        const SIZEOF_SOCKLEN_T: usize = size_of::<libc::socklen_t>();
         let mut buf = [0u8; SIZEOF_SOCKLEN_T];
         if request.read_mem(&mut buf, args[2])? == SIZEOF_SOCKLEN_T {
             // libc defines socklen_t as u32,
@@ -43,7 +45,7 @@ pub(crate) fn handle_getsockname(
             // and return EINVAL as necessary.
             let len = i32::from_ne_bytes(buf);
             let len = libc::socklen_t::try_from(len).or(Err(Errno::EINVAL))?;
-            if args[1] == 0 {
+            if args[1] < *MMAP_MIN_ADDR {
                 // address length is positive however address is NULL,
                 // return EFAULT.
                 return Err(Errno::EFAULT);
