@@ -8,7 +8,6 @@
 
 use std::{
     borrow::Cow,
-    ffi::CString,
     net::IpAddr,
     ops::Deref,
     os::{
@@ -21,26 +20,32 @@ use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
     fcntl::OFlag,
-    sys::socket::{AddressFamily, MsgFlags, SockaddrLike, SockaddrStorage},
+    sys::socket::{AddressFamily, SockaddrLike, SockaddrStorage, UnixAddr},
     NixPath,
 };
 
 use crate::{
-    compat::{addr_family, PF_ALG, PF_INET, PF_INET6, PF_MAX, PF_NETLINK, PF_UNIX, PF_UNSPEC},
+    compat::{
+        addr_family, MsgFlags, PF_ALG, PF_INET, PF_INET6, PF_MAX, PF_NETLINK, PF_UNIX, PF_UNSPEC,
+    },
+    config::HOOK_SCKCALLS,
     confine::{op2errno, op2name, scmp_arch_bits},
     fs::{fd_status_flags, file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
-    hook::UNotifyEventRequest,
     kernel::net::{
         accept::handle_accept,
         bind::handle_bind,
         connect::handle_connect,
+        getpeername::handle_getpeername,
         getsockname::handle_getsockname,
         getsockopt::handle_getsockopt,
+        recvfrom::handle_recvfrom,
+        recvmsg::{handle_recvmmsg, handle_recvmmsg64, handle_recvmsg},
         sendmsg::{handle_sendmmsg, handle_sendmsg},
         sendto::handle_sendto,
-        socket::handle_socket,
+        socket::{handle_socket, handle_socketpair},
     },
     path::{XPath, XPathBuf},
+    req::UNotifyEventRequest,
     sandbox::{Action, Capability, SandboxGuard},
     warn,
 };
@@ -48,8 +53,11 @@ use crate::{
 pub(crate) mod accept;
 pub(crate) mod bind;
 pub(crate) mod connect;
+pub(crate) mod getpeername;
 pub(crate) mod getsockname;
 pub(crate) mod getsockopt;
+pub(crate) mod recvfrom;
+pub(crate) mod recvmsg;
 pub(crate) mod sendmsg;
 pub(crate) mod sendto;
 pub(crate) mod socket;
@@ -59,27 +67,15 @@ const UNIX_PATH_MAX: usize = 108;
 pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // Determine system call
-    // 0x1 => socket
-    // 0x2 => bind
-    // 0x3 => connect
-    // 0x5 => accept
-    // 0x6 => getsockname
-    // 0x9 => send
-    // 0xb => sendto
-    // 0xf => getsockopt
-    // 0x10 => sendmsg
-    // 0x12 => accept4
-    // 0x14 => sendmmsg
-    #[expect(clippy::cast_possible_truncation)]
-    let op = match req.data.args[0] {
-        n @ (0x1 | 0x2 | 0x3 | 0x5 | 0x6 | 0x9 | 0xb | 0xf | 0x10 | 0x12 | 0x14) => n as u8,
-        _ => {
-            // SAFETY: Safe network call, continue.
-            // No pointer-dereference in access check.
-            return unsafe { request.continue_syscall() };
-        }
+    // Determine socket subcall.
+    let op: u8 = match req.data.args[0].try_into() {
+        Ok(op) => op,
+        Err(_) => return request.fail_syscall(Errno::EINVAL),
     };
+    if HOOK_SCKCALLS.binary_search(&op).is_err() {
+        // SAFETY: No pointer dereference in access check.
+        return unsafe { request.continue_syscall() };
+    }
 
     // Determine system call arguments.
     // On x86 unsigned long is 4 bytes, and on s390x 8 bytes.
@@ -128,6 +124,11 @@ pub(crate) fn sys_socket(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x1)
 }
 
+pub(crate) fn sys_socketpair(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0x8)
+}
+
 pub(crate) fn sys_bind(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     syscall_network_handler(request, &req.data.args, 0x2)
@@ -143,6 +144,11 @@ pub(crate) fn sys_accept4(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x12)
 }
 
+pub(crate) fn sys_getpeername(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0x7)
+}
+
 pub(crate) fn sys_getsockname(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     syscall_network_handler(request, &req.data.args, 0x6)
@@ -158,6 +164,28 @@ pub(crate) fn sys_connect(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x3)
 }
 
+pub(crate) fn sys_recvfrom(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0xc)
+}
+
+pub(crate) fn sys_recvmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0x11)
+}
+
+pub(crate) fn sys_recvmmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0x13)
+}
+
+pub(crate) fn sys_recvmmsg64(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    // recvmmsg_time64 does not have a socketcall(2) equivalent,
+    // therefore we use the dummy op `u8::MAX`.
+    syscall_network_handler(request, &req.data.args, u8::MAX)
+}
+
 pub(crate) fn sys_sendto(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     syscall_network_handler(request, &req.data.args, 0xb)
@@ -178,27 +206,34 @@ pub(crate) fn sys_sendmmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
 fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let sandbox = request.get_sandbox();
-        let allow_safe_bind = sandbox.flags.allow_safe_bind();
-        let allow_safe_kcapi = sandbox.flags.allow_safe_kcapi();
-        let allow_unsupp_socket = sandbox.flags.allow_unsupp_socket();
-        let restrict_oob = !sandbox.flags.allow_unsafe_oob();
-        let restrict_mkbdev = !sandbox.flags.allow_unsafe_mkbdev();
+        let flags = *sandbox.flags;
+        let allow_safe_bind = flags.allow_safe_bind();
+        let allow_safe_kcapi = flags.allow_safe_kcapi();
+        let allow_unsupp_socket = flags.allow_unsupp_socket();
+        let restrict_oob = !flags.allow_unsafe_oob();
 
         let cap = match op {
             0x1 => {
                 // a. socket system call.
                 // b. socketcall -> socket indirection.
-                let flags = *sandbox.flags;
                 let nlfam = sandbox.netlink_families;
                 drop(sandbox); // drop read-lock before emulation.
-
                 return handle_socket(&request, args, flags, nlfam);
             }
-            0x5 | 0x6 | 0x12 => {
-                // accept, accept4, getsockname:
+            0x8 => {
+                // a. socketpair system call.
+                // b. socketcall -> socketpair indirection.
+                // As of version 3.42.0, this is checked for bind
+                // access with the dummy `!unnamed' path.
+                return handle_socketpair(&request, sandbox, args, 0x8);
+            }
+            0x5 | 0x6 | 0x7 | 0xc | 0x11 | 0x12 | 0x13 | u8::MAX => {
+                // accept, accept4, getsockname, getpeername:
                 //
                 // accept{,4} are IP blocklist only.
-                // getsockname is informational.
+                // get{peer,sock}name are informational.
+                // recvfrom is informational.
+                // recvmsg, recvmmsg, and recvmmsg_time64 are informational.
                 Capability::empty()
             }
             0x2 /* bind */ => Capability::CAP_NET_BIND,
@@ -234,37 +269,43 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // getsockname is used for informational purposes only.
                 return handle_getsockname(fd, &request, args);
             }
+            0x7 => {
+                // getpeername is used for informational purposes only.
+                return handle_getpeername(fd, &request, args);
+            }
             0x9 => {
                 // send: Connection mode socket.
                 // This only happens via socketcall(2) multiplexer.
                 // On 64-bit we do not hook into send(2).
                 return handle_sendto(fd, args, &request, None, restrict_oob);
             }
+            0xc => {
+                // recvfrom is used for informational purposes only.
+                return handle_recvfrom(fd, args, &request, restrict_oob);
+            }
+            0x11 => {
+                // recvmsg is used for informational purposes only.
+                return handle_recvmsg(fd, &request, args, flags);
+            }
+            0x13 => {
+                // recvmsg is used for informational purposes only.
+                return handle_recvmmsg(fd, &request, args, flags);
+            }
+            u8::MAX => {
+                // recvmmsg_time64 is used for informational purposes only.
+                return handle_recvmmsg64(fd, &request, args, flags);
+            }
             0xf => {
                 // getsockopt is used for informational purposes only.
                 return handle_getsockopt(fd, &request, args);
             }
             0x10 => {
                 // sendmsg uses a different data structure, so we handle it in its own branch.
-                return handle_sendmsg(
-                    fd,
-                    &request,
-                    args,
-                    allow_unsupp_socket,
-                    restrict_oob,
-                    restrict_mkbdev,
-                );
+                return handle_sendmsg(fd, &request, args, flags);
             }
             0x14 => {
                 // sendmmsg uses a different data structure, so we handle it in its own branch.
-                return handle_sendmmsg(
-                    fd,
-                    &request,
-                    args,
-                    allow_unsupp_socket,
-                    restrict_oob,
-                    restrict_mkbdev,
-                );
+                return handle_sendmmsg(fd, &request, args, flags);
             }
             _ => {} // fall through.
         }
@@ -286,12 +327,17 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
         } // else we have a valid address to check for access.
 
         let sandbox = request.get_sandbox();
-        let (addr, root) = canon_addr(
-            &request,
-            &sandbox,
-            get_addr(&request, addr_remote, addr_len)?,
-            cap,
-        )?;
+        let argaddr = if op == 0x2 && addr_len as usize == size_of::<libc::sa_family_t>() {
+            // unix(7): "If a bind(2) call specifies addrlen as `sizeof(sa_family_t)`,
+            // or [...], then the socket is autobound to an abstract address".
+            let addr = UnixAddr::new_unnamed();
+            // SAFETY: `addr` is constructed using new_unnamed.
+            #[expect(clippy::disallowed_methods)]
+            unsafe { SockaddrStorage::from_raw(addr.as_ptr().cast(), Some(addr.len())) }.unwrap()
+        } else {
+            get_addr(&request, addr_remote, addr_len)?
+        };
+        let (addr, root) = canon_addr(&request, &sandbox, &argaddr, cap)?;
         match addr_family(&addr) {
             PF_UNIX | PF_INET | PF_INET6 => {
                 // Check for access.
@@ -326,9 +372,9 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
         // Emulate syscall.
         match op {
-            0x2 => handle_bind(fd, &addr, root.as_ref(), &request, allow_safe_bind),
-            0x3 => handle_connect(fd, &addr, &request, allow_safe_bind),
-            0xb => handle_sendto(fd, args, &request, Some(&addr), restrict_oob),
+            0x2 => handle_bind(fd, (addr, argaddr), root, &request, allow_safe_bind),
+            0x3 => handle_connect(fd, (addr, argaddr), &request, allow_safe_bind),
+            0xb => handle_sendto(fd, args, &request, Some(addr), restrict_oob),
             _ => unreachable!(),
         }
     })
@@ -347,7 +393,7 @@ fn get_addr(
         return Err(Errno::EINVAL);
     }
     #[expect(clippy::arithmetic_side_effects)]
-    let addr_len = addr_len.min(std::mem::size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX);
+    let addr_len = addr_len.min(size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX);
 
     let mut addr = Vec::new();
     addr.try_reserve(addr_len).or(Err(Errno::ENOMEM))?;
@@ -364,13 +410,13 @@ fn get_addr(
     match unsafe { SockaddrStorage::from_raw(addr, Some(addr_len)) } {
         Some(addr)
             if addr.as_sockaddr_in().is_some()
-                && (addr_len as usize) < std::mem::size_of::<libc::sockaddr_in>() =>
+                && (addr_len as usize) < size_of::<libc::sockaddr_in>() =>
         {
             Err(Errno::EINVAL)
         }
         Some(addr)
             if addr.as_sockaddr_in6().is_some()
-                && (addr_len as usize) < std::mem::size_of::<libc::sockaddr_in6>() =>
+                && (addr_len as usize) < size_of::<libc::sockaddr_in6>() =>
         {
             Err(Errno::EINVAL)
         }
@@ -388,7 +434,7 @@ fn get_addr(
 fn canon_addr<'a>(
     request: &UNotifyEventRequest,
     sandbox: &SandboxGuard,
-    addr: SockaddrStorage,
+    addr: &SockaddrStorage,
     cap: Capability,
 ) -> Result<(SockaddrStorage, Option<CanonicalPath<'a>>), Errno> {
     #[expect(clippy::cast_possible_truncation)]
@@ -420,7 +466,7 @@ fn canon_addr<'a>(
         let pid = request.scmpreq.pid();
         let path = safe_canonicalize(pid, None, &path, fsflags, Some(sandbox.deref()))?;
 
-        let cstr = if path.base.is_empty() {
+        let path_bytes = if path.base.is_empty() {
             // SAFETY: We open a FD to the path and then use the
             // proc path /proc/thread-self/fd/$fd in address' path
             // argument to avoid symlink TOCTOU because connect and
@@ -429,24 +475,30 @@ fn canon_addr<'a>(
             let fd = path.dir.as_ref().unwrap();
             let mut pfd = XPathBuf::from("/proc/thread-self/fd");
             pfd.push_fd(fd.as_raw_fd());
-            CString::new(pfd.as_os_str().as_bytes())
+            pfd.append_byte(0);
+            pfd.into_vec()
         } else {
-            // SAFETY: We split the address into directory and basename
-            // regardless of UNIX_PATH_MAX as we are later going to use
-            // the handler thread to mitigate the TOCTOU vector in the
-            // basename of the UNIX socket address. This is only used
-            // for bind() which does not resolve symbolic links in
-            // basename.
-            CString::new(path.base.as_os_str().as_bytes())
-        }
-        .or(Err(Errno::EINVAL))?;
+            // SAFETY:
+            // 1. We split the address into directory and basename
+            //    regardless of UNIX_PATH_MAX as we are later going to use
+            //    the handler thread to mitigate the TOCTOU vector in the
+            //    basename of the UNIX socket address. This is only used
+            //    for bind() which does not resolve symbolic links in
+            //    basename.
+            // 2. We add "./" for easier identification at recvfrom(2)
+            //    boundary which is for informational purposes only.
+            let mut base = XPathBuf::from("./");
+            base.append_bytes(path.base.as_os_str().as_bytes());
+            base.append_byte(0);
+            base.into_vec()
+        };
 
         // Create sockaddr_un struct.
         let mut sockaddr = libc::sockaddr_un {
             sun_family: libc::AF_UNIX as libc::sa_family_t,
             sun_path: [0; UNIX_PATH_MAX],
         };
-        let socklen = cstr.as_bytes_with_nul().len();
+        let socklen = path_bytes.len();
         if socklen > UNIX_PATH_MAX {
             return Err(Errno::ENAMETOOLONG);
         }
@@ -454,7 +506,7 @@ fn canon_addr<'a>(
         // SAFETY: Copy the bytes without overlapping regions.
         unsafe {
             std::ptr::copy_nonoverlapping(
-                cstr.as_ptr(),
+                path_bytes.as_ptr(),
                 sockaddr.sun_path.as_mut_ptr().cast(),
                 socklen,
             )
@@ -465,7 +517,7 @@ fn canon_addr<'a>(
         // the sun_path field plus the length of the path (including the
         // null terminator).
         #[expect(clippy::arithmetic_side_effects)]
-        let size = std::mem::size_of::<libc::sa_family_t>() + socklen;
+        let size = size_of::<libc::sa_family_t>() + socklen;
 
         // SAFETY: We are converting a sockaddr_un to a
         // SockaddrStorage using a raw pointer. The sockaddr_un
@@ -483,7 +535,7 @@ fn canon_addr<'a>(
         Ok((addr, Some(path)))
     } else {
         // No need to canonicalize.
-        Ok((addr, None))
+        Ok((*addr, None))
     }
 }
 
@@ -734,7 +786,59 @@ pub(crate) fn sandbox_addr_notsup(sandbox: &SandboxGuard) -> Result<(), Errno> {
     }
 }
 
-#[inline]
+/// Process an unnamed socket with the given socket family.
+///
+/// Used at socketpair(2) boundary for AF_UNIX sockets.
+#[expect(clippy::cognitive_complexity)]
+pub(crate) fn sandbox_addr_unnamed(
+    request: &UNotifyEventRequest,
+    sandbox: &SandboxGuard,
+    op: u8,
+) -> Result<(), Errno> {
+    // socketpair
+    assert_eq!(
+        op, 0x8,
+        "BUG: sandbox_addr_unnamed called with invalid op:{op:#x}, report a bug!"
+    );
+
+    // Check for access.
+    let caps = Capability::CAP_NET_BIND;
+    let name = XPath::from_bytes(b"!unnamed");
+
+    if sandbox.getcaps(caps).is_empty() {
+        // Sandboxing is off.
+        return Ok(());
+    }
+    let (action, filter) = sandbox.check_unix(caps, name);
+
+    if !filter {
+        let sys = op2name(op);
+        if sandbox.verbose {
+            warn!("ctx": "access", "cap": caps, "act": action,
+                "sys": sys, "unix": &name, "abs": true,
+                "tip": format!("configure `allow/{caps}+{name}'"),
+                "req": request);
+        } else {
+            warn!("ctx": "access", "cap": caps, "act": action,
+                "sys": sys, "unix": &name, "abs": true,
+                "tip": format!("configure `allow/{caps}+{name}'"),
+                "pid": request.scmpreq.pid);
+        }
+    }
+
+    match action {
+        Action::Allow | Action::Warn => Ok(()),
+        Action::Deny | Action::Filter => Err(op2errno(op)),
+        Action::Panic => panic!(),
+        Action::Exit => std::process::exit(op2errno(op) as i32),
+        action => {
+            // Stop|Kill
+            let _ = request.kill(action);
+            Err(op2errno(op))
+        }
+    }
+}
+
 pub(crate) fn to_msgflags(arg: u64) -> Result<MsgFlags, Errno> {
     let flags = arg.try_into().or(Err(Errno::EINVAL))?;
     MsgFlags::from_bits(flags).ok_or(Errno::EINVAL)
