@@ -9,9 +9,13 @@
 use std::{
     borrow::Cow,
     env,
+    ffi::CStr,
     fs::File,
     io::{BufWriter, Seek, Write},
-    os::fd::{AsFd, AsRawFd, IntoRawFd, OwnedFd, RawFd},
+    os::{
+        fd::{AsFd, AsRawFd, IntoRawFd, OwnedFd, RawFd},
+        unix::ffi::OsStrExt,
+    },
 };
 
 use libseccomp::ScmpNotifResp;
@@ -45,7 +49,7 @@ use crate::{
     proc::{proc_tty, proc_umask},
     req::{SysArg, UNotifyEventRequest},
     retry::retry_on_eintr,
-    sandbox::{Action, Capability},
+    sandbox::{Action, Capability, Sandbox},
     syslog::LogLevel,
     warn,
     workers::aes::{AesMod, AesVal},
@@ -152,17 +156,12 @@ pub(crate) fn sys_openat2(request: UNotifyEventRequest) -> ScmpNotifResp {
         Err(errno) => return request.fail_syscall(errno),
     };
 
-    // SAFETY:
-    // 1. Return ENOSYS for valid but unsupported openat2 resolve flags.
-    // 2. Return EINVAL for invalid resolve flags.
+    // SAFETY: Return EINVAL for invalid resolve flags or combinations.
     let rflags = match ResolveFlag::from_bits(open_how.resolve) {
-        Some(rflags) if rflags.contains(ResolveFlag::RESOLVE_IN_ROOT) => {
-            let errno = if rflags.contains(ResolveFlag::RESOLVE_BENEATH) {
-                Errno::EINVAL
-            } else {
-                Errno::ENOSYS
-            };
-            return request.fail_syscall(errno);
+        Some(rflags)
+            if rflags.contains(ResolveFlag::RESOLVE_BENEATH | ResolveFlag::RESOLVE_IN_ROOT) =>
+        {
+            return request.fail_syscall(Errno::EINVAL);
         }
         Some(rflags) => rflags,
         None => return request.fail_syscall(Errno::EINVAL),
@@ -599,6 +598,7 @@ fn syscall_open_handler(
         let restrict_create = !sandbox.flags.allow_unsafe_create();
         let filter_machine_id = !sandbox.flags.allow_unsafe_machine_id();
         let filter_proc_status = !sandbox.flags.allow_unsafe_proc_pid_status();
+        let filter_uname = !sandbox.flags.allow_unsafe_uname();
         if sandbox.flags.force_cloexec() {
             o_cloexec = true;
         }
@@ -826,7 +826,11 @@ fn syscall_open_handler(
                     flags.remove(OFlag::O_CREAT);
                     flags.remove(OFlag::O_TMPFILE);
                 }
-                flags.insert(OFlag::O_NOFOLLOW);
+
+                // We use RESOLVE_NO_SYMLINKS to prevent symlinks as needed,
+                // and do not want to leak O_NOFOLLOW into F_GETFL(2const).
+                // See Linux kernel's openat2 tests for more information.
+                // flags.insert(OFlag::O_NOFOLLOW);
 
                 let fd: OwnedFd = if crypted {
                     // Handle Encryption.
@@ -849,120 +853,16 @@ fn syscall_open_handler(
                         return Ok(ScmpNotifResp::new(0, 0, EIDRM, 0));
                     }
                 } else if path.base.is_empty() && !o_tmpfl {
-                    // Existing path: Construct path to proc(5) magic symlink,
-                    // or pre-open file descriptor.
-                    //
-                    // SAFETY: Note, the path may be borrowed here, ie
-                    // it can be a fd to the preopen `/`, `/proc` or
-                    // `/dev/null` fds. In these cases, we still want to
-                    // reopen, because the fd we're going to send will
-                    // share the same open file description and we do
-                    // not want to mess up file offsets for everyone.
-                    //
-                    // SAFETY: ^^ empty base asserts dir is Some.
-                    #[expect(clippy::disallowed_methods)]
-                    let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
-                    let pfd = XPathBuf::from_self_fd(fd);
-
-                    // Reopen the `O_PATH` path fd with the requested flags.
-                    flags.remove(OFlag::O_CREAT);
-                    flags.remove(OFlag::O_NOFOLLOW);
-                    let how = safe_open_how_msym(flags, ResolveFlag::empty());
-
-                    // SAFETY: Record blocking call so it can get invalidated.
-                    if is_blocking {
-                        request.cache.add_sys_block(req, false)?;
+                    if let Some(fd) = do_open_pre(&request, &path, filter_machine_id, filter_uname)?
+                    {
+                        // pre-open hook intercepted, no need to open.
+                        fd
+                    } else {
+                        // Existing path: Construct path to proc(5) magic symlink,
+                        // or pre-open file descriptor.
+                        let fd = do_open(&request, &path, flags, is_blocking)?;
+                        do_open_post(fd, &path, filter_proc_status)?
                     }
-
-                    let result = safe_openat2(PROC_FILE(), &pfd, how);
-
-                    // Remove invalidation record unless interrupted.
-                    if is_blocking {
-                        request
-                            .cache
-                            .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
-                    }
-
-                    let mut fd = result?;
-
-                    if filter_machine_id && path.abs().is_machine_id() {
-                        // SAFETY: Substitute machine-id(5) with per-Syd random boot-id.
-                        //
-                        // Generate machine-id(5) contents.
-                        #[expect(clippy::disallowed_methods)]
-                        let mut machine_id =
-                            env::var(ENV_ID).expect("BUG: SYD_ID environment variable isn't set!");
-                        machine_id.truncate(32);
-                        machine_id.push('\n');
-
-                        // Open a memory fd to be sealed for write.
-                        // Memory fd is exec-sealed on Linux>=6.3.
-                        let mut mfd = safe_memfd_create(c"syd-machine-id", *SAFE_MFD_FLAGS)
-                            .map(File::from)
-                            .map(BufWriter::new)?;
-
-                        // Write file contents, retrying on interrupts.
-                        mfd.write_all(machine_id.as_bytes())
-                            .map_err(|err| err2no(&err))?;
-                        // into_inner fails on flush errors.
-                        let mut mfd = mfd.into_inner().or(Err(Errno::EIO))?;
-
-                        // Rewind file offset and seal to deny further writes.
-                        mfd.rewind().or(Err(Errno::EIO))?;
-                        seal_memfd(&mfd)?;
-
-                        fd = mfd.into();
-                    } else if filter_proc_status && path.abs().is_proc_pid_status() {
-                        // SAFETY: Filter security-sensitive proc_pid_status(5) contents.
-                        //
-                        // Open a memory fd to be sealed for write.
-                        // Memory fd is exec-sealed on Linux>=6.3.
-                        let mut mfd = safe_memfd_create(c"syd-proc-pid-status", *SAFE_MFD_FLAGS)
-                            .map(File::from)?;
-
-                        // Filter proc_pid_status(5) into the memory fd.
-                        mask_proc_pid_status(fd, &mfd)?;
-
-                        // Rewind file offset and seal to deny further writes.
-                        mfd.rewind().or(Err(Errno::EIO))?;
-                        seal_memfd(&mfd)?;
-
-                        fd = mfd.into();
-                    } else if path.abs().is_proc_version() {
-                        // SAFETY: Provide identical data to uname(2).
-                        //
-                        // Open a memory fd to be sealed for write.
-                        // Memory fd is exec-sealed on Linux>=6.3.
-                        let mut mfd = safe_memfd_create(c"syd-proc-version", *SAFE_MFD_FLAGS)
-                            .map(File::from)
-                            .map(BufWriter::new)?;
-
-                        // Read UtsName using uname(2) syscall.
-                        //
-                        // SAFETY: Syscall is confined with syscall argument cookies.
-                        let mut utsname = safe_uname()?;
-
-                        // Mask utsname using the sandbox.
-                        let sandbox = request.get_sandbox();
-                        sandbox.mask_uname(&mut utsname);
-                        drop(sandbox); // release the read-lock.
-
-                        // Write the utsname's Display into file,
-                        // retry on interrupts.
-                        let utsname = utsname.to_string();
-                        mfd.write_all(utsname.as_bytes())
-                            .map_err(|err| err2no(&err))?;
-                        // into_inner fails on flush errors.
-                        let mut mfd = mfd.into_inner().or(Err(Errno::EIO))?;
-
-                        // Rewind file offset and seal to deny further writes.
-                        mfd.rewind().or(Err(Errno::EIO))?;
-                        seal_memfd(&mfd)?;
-
-                        fd = mfd.into();
-                    }
-
-                    fd
                 } else if o_mask {
                     // Cannot be O_PATH or encrypted fd!
                     //
@@ -1264,7 +1164,177 @@ fn do_open_crypt(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd,
     safe_open_msym(PROC_FILE(), &pfd, safe_flags, ResolveFlag::empty())
 }
 
-#[inline]
+// Existing path: Construct path to proc(5) magic symlink, or pre-open file descriptor.
+fn do_open(
+    request: &UNotifyEventRequest,
+    path: &CanonicalPath,
+    mut flags: OFlag,
+    is_blocking: bool,
+) -> Result<OwnedFd, Errno> {
+    assert!(path.base.is_empty());
+
+    // SAFETY: Note, the path may be borrowed here, ie it can be a fd to the preopen `/`,
+    // `/proc`, or `/dev/null` fds. In these cases, we still want to reopen, because the
+    // fd we're going to send will share the same open file description and we don't want
+    // to mess up file offsets for everyone.
+    // SAFETY: ^^ empty base asserts dir is Some.
+    #[expect(clippy::disallowed_methods)]
+    let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
+    let pfd = XPathBuf::from_self_fd(fd);
+
+    // Reopen the `O_PATH` path fd with the requested flags.
+    flags.remove(OFlag::O_CREAT);
+    flags.remove(OFlag::O_NOFOLLOW);
+    let how = safe_open_how_msym(flags, ResolveFlag::empty());
+
+    // SAFETY: Record blocking call so it can get invalidated.
+    let req = request.scmpreq;
+    if is_blocking {
+        request.cache.add_sys_block(req, false)?;
+    }
+
+    let result = safe_openat2(PROC_FILE(), &pfd, how);
+
+    // Remove invalidation record unless interrupted.
+    if is_blocking {
+        request
+            .cache
+            .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
+    }
+
+    result
+}
+
+fn do_open_pre(
+    request: &UNotifyEventRequest,
+    path: &CanonicalPath,
+    filter_machine_id: bool,
+    filter_uname: bool,
+) -> Result<Option<OwnedFd>, Errno> {
+    // SAFETY: Substitute machine-id(5) with per-Syd random boot-id.
+    if filter_machine_id && path.abs().is_machine_id() {
+        return Ok(Some(do_open_machine_id()?));
+    }
+
+    // SAFETY: Provide identical data to uname(2) for /proc/version.
+    if filter_uname && path.abs().is_proc_version() {
+        return Ok(Some(do_open_proc_version(request)?));
+    }
+
+    // SAFETY: Provide identical data to uname(2) for /proc/sys/kernel/osrelease.
+    if filter_uname && path.abs().is_proc_osrelease() {
+        return Ok(Some(do_open_proc_osrelease()?));
+    }
+
+    Ok(None)
+}
+
+fn do_open_post(
+    mut fd: OwnedFd,
+    path: &CanonicalPath,
+    filter_proc_status: bool,
+) -> Result<OwnedFd, Errno> {
+    // SAFETY: Filter security-sensitive proc_pid_status(5) contents.
+    if filter_proc_status && path.abs().is_proc_pid_status() {
+        fd = do_mask_proc_pid_status(fd)?;
+    }
+
+    Ok(fd)
+}
+
+fn do_open_machine_id() -> Result<OwnedFd, Errno> {
+    // Open a memory fd to be sealed for write.
+    let mut mfd = do_open_memfd(c"syd-machine-id").map(BufWriter::new)?;
+
+    // Generate machine-id(5) contents.
+    #[expect(clippy::disallowed_methods)]
+    let mut machine_id = env::var(ENV_ID).expect("BUG: SYD_ID environment variable isn't set!");
+    machine_id.truncate(32);
+    machine_id.push('\n');
+
+    // Write file contents, retrying on interrupts.
+    mfd.write_all(machine_id.as_bytes())
+        .map_err(|err| err2no(&err))?;
+    // into_inner fails on flush errors.
+    let mut mfd = mfd.into_inner().or(Err(Errno::EIO))?;
+
+    // Rewind file offset and seal to deny further writes.
+    mfd.rewind().or(Err(Errno::EIO))?;
+    seal_memfd(&mfd)?;
+
+    Ok(mfd.into())
+}
+
+fn do_open_proc_version(request: &UNotifyEventRequest) -> Result<OwnedFd, Errno> {
+    // Open a memory fd to be sealed for write.
+    let mut mfd = do_open_memfd(c"syd-proc-version").map(BufWriter::new)?;
+
+    // Read UtsName using uname(2) syscall.
+    //
+    // SAFETY: Syscall is confined with syscall argument cookies.
+    let mut utsname = safe_uname()?;
+
+    // Mask utsname using the sandbox.
+    let sandbox = request.get_sandbox();
+    sandbox.mask_uname(&mut utsname);
+    drop(sandbox); // release the read-lock.
+
+    // Write the utsname's Display into file, retry on interrupts.
+    let utsname = utsname.to_string();
+    mfd.write_all(utsname.as_bytes())
+        .map_err(|err| err2no(&err))?;
+    // into_inner fails on flush errors.
+    let mut mfd = mfd.into_inner().or(Err(Errno::EIO))?;
+
+    // Rewind file offset and seal to deny further writes.
+    mfd.rewind().or(Err(Errno::EIO))?;
+    seal_memfd(&mfd)?;
+
+    Ok(mfd.into())
+}
+
+fn do_open_proc_osrelease() -> Result<OwnedFd, Errno> {
+    // Open a memory fd to be sealed for write.
+    let mut mfd = do_open_memfd(c"syd-proc-sys-kernel-osrelease").map(BufWriter::new)?;
+
+    // Read static release string.
+    let release = Sandbox::os_release();
+
+    // Write release string avoiding interrupts.
+    // End it with a newline.
+    mfd.write_all(release.as_bytes())
+        .map_err(|err| err2no(&err))?;
+    mfd.write_all(b"\n").map_err(|err| err2no(&err))?;
+    // into_inner fails on flush errors.
+    let mut mfd = mfd.into_inner().or(Err(Errno::EIO))?;
+
+    // Rewind file offset and seal to deny further writes.
+    mfd.rewind().or(Err(Errno::EIO))?;
+    seal_memfd(&mfd)?;
+
+    Ok(mfd.into())
+}
+
+fn do_mask_proc_pid_status(fd: OwnedFd) -> Result<OwnedFd, Errno> {
+    // Open a memory fd to be sealed for write.
+    let mut mfd = do_open_memfd(c"syd-proc-pid-status")?;
+
+    // Filter proc_pid_status(5) into the memory fd.
+    mask_proc_pid_status(fd, &mfd)?;
+
+    // Rewind file offset and seal to deny further writes.
+    mfd.rewind().or(Err(Errno::EIO))?;
+    seal_memfd(&mfd)?;
+
+    Ok(mfd.into())
+}
+
+// Open a memory fd to be sealed for write.
+// Memory fd is exec-sealed on Linux>=6.3.
+fn do_open_memfd(name: &CStr) -> Result<File, Errno> {
+    safe_memfd_create(name, *SAFE_MFD_FLAGS).map(File::from)
+}
+
 fn to_mode(arg: u64, flags: OFlag, ignore: bool) -> Result<Mode, Errno> {
     if flags.contains(OFlag::O_CREAT) || flags.contains(OFlag::O_TMPFILE) {
         if ignore {
