@@ -15,8 +15,8 @@ use crate::{
     compat::RenameFlags,
     cookie::safe_renameat2,
     fs::FsFlags,
-    hook::{PathArgs, SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
+    req::{PathArgs, SysArg, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_rename(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -115,9 +115,9 @@ fn syscall_rename_handler(
     flags: RenameFlags,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: SysArg has two elements.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let old_path = args.0.as_ref().unwrap();
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let new_path = args.1.as_ref().unwrap();
 
     safe_renameat2(
