@@ -435,7 +435,7 @@ fn syscall_stat_handler(
             // SAFETY: The following block creates an immutable byte
             // slice representing the memory of `statx`. We ensure that
             // the slice covers the entire memory of `statx` using
-            // `std::mem::size_of_val`. Since `statx` is a stack
+            // `size_of_val`. Since `statx` is a stack
             // variable and we're only borrowing its memory for the
             // duration of the slice, there's no risk of `statx` being
             // deallocated while the slice exists. Additionally, we
@@ -444,7 +444,7 @@ fn syscall_stat_handler(
             let statx = unsafe {
                 std::slice::from_raw_parts(
                     std::ptr::addr_of!(statx) as *const u8,
-                    std::mem::size_of_val(&statx),
+                    size_of_val(&statx),
                 )
             };
             let addr = req.data.args[4];
@@ -482,7 +482,7 @@ fn syscall_stat_handler(
                     // SAFETY: The following block creates an immutable
                     // byte slice representing the memory of `stat`.  We
                     // ensure that the slice covers the entire memory of
-                    // `stat` using `std::mem::size_of_val`. Since
+                    // `stat` using `size_of_val`. Since
                     // `stat` is a stack variable and we're only
                     // borrowing its memory for the duration of the
                     // slice, there's no risk of `stat` being
@@ -492,7 +492,7 @@ fn syscall_stat_handler(
                     let stat = unsafe {
                         std::slice::from_raw_parts(
                             std::ptr::addr_of!(stat32) as *const u8,
-                            std::mem::size_of_val(&stat32),
+                            size_of_val(&stat32),
                         )
                     };
                     request.write_mem(stat, addr)?;
@@ -500,7 +500,7 @@ fn syscall_stat_handler(
                     // SAFETY: The following block creates an immutable
                     // byte slice representing the memory of `stat`.  We
                     // ensure that the slice covers the entire memory of
-                    // `stat` using `std::mem::size_of_val`. Since
+                    // `stat` using `size_of_val`. Since
                     // `stat` is a stack variable and we're only
                     // borrowing its memory for the duration of the
                     // slice, there's no risk of `stat` being
@@ -510,7 +510,7 @@ fn syscall_stat_handler(
                     let stat = unsafe {
                         std::slice::from_raw_parts(
                             std::ptr::addr_of!(stat) as *const u8,
-                            std::mem::size_of_val(&stat),
+                            size_of_val(&stat),
                         )
                     };
                     request.write_mem(stat, addr)?;
