@@ -44,7 +44,7 @@ pub(crate) fn sys_statfs(request: UNotifyEventRequest) -> ScmpNotifResp {
             .as_ref()
             .ok_or(Errno::EACCES)?;
 
-        const SIZ: usize = std::mem::size_of::<libc::statfs>();
+        const SIZ: usize = size_of::<libc::statfs>();
         let mut buf: Vec<u8> = Vec::new();
         buf.try_reserve(SIZ).or(Err(Errno::ENOMEM))?;
         buf.resize(SIZ, 0);
@@ -152,7 +152,7 @@ pub(crate) fn sys_fstatfs(request: UNotifyEventRequest) -> ScmpNotifResp {
         #[expect(clippy::disallowed_methods)]
         let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
-        const SIZ: usize = std::mem::size_of::<libc::statfs>();
+        const SIZ: usize = size_of::<libc::statfs>();
         let mut buf: Vec<u8> = Vec::new();
         buf.try_reserve(SIZ).or(Err(Errno::ENOMEM))?;
         buf.resize(SIZ, 0);
