@@ -8,16 +8,17 @@
 
 use std::os::fd::AsRawFd;
 
+use libc::c_int;
 use libseccomp::{ScmpArch, ScmpNotifResp};
 use nix::{errno::Errno, fcntl::FallocateFlags};
 
 use crate::{
+    confine::{scmp_arch_bits, scmp_big_endian},
     cookie::{safe_ftruncate, safe_ftruncate64, safe_truncate, safe_truncate64},
-    fs::is_valid_fd,
-    hook::{SysArg, UNotifyEventRequest},
+    fs::{fallocate64, is_valid_fd},
     kernel::syscall_path_handler,
     path::XPathBuf,
-    scmp_arch_bits, scmp_big_endian,
+    req::{SysArg, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_truncate(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -27,9 +28,8 @@ pub(crate) fn sys_truncate(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
-    #[allow(clippy::cast_lossless)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         libc::off_t::from(req.data.args[1] as i32)
     } else {
@@ -53,7 +53,7 @@ pub(crate) fn sys_truncate(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. `/` is not a regular file -> EINVAL.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args
             .0
             .as_ref()
@@ -89,20 +89,23 @@ pub(crate) fn sys_truncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     // truncate64 needs argument translation on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         // Align arg_no to the next even number for specific 32-bit architectures.
         let arg_no = match req.data.arch {
-            ScmpArch::Arm | ScmpArch::Ppc => 2,
+            ScmpArch::Arm | ScmpArch::Mipsel | ScmpArch::Ppc => 2,
             _ => 1,
         };
 
         let len_low = req.data.args[arg_no] as u32;
         let len_high = req.data.args[arg_no + 1] as u32;
 
-        if scmp_big_endian(req.data.arch) {
+        if req.data.arch == ScmpArch::Ppc && cfg!(target_pointer_width = "32") {
+            // PowerPC32 passes high word first, then low word.
+            (libc::off64_t::from(len_high) << 32) | libc::off64_t::from(len_low)
+        } else if scmp_big_endian(req.data.arch) {
             (libc::off64_t::from(len_low) << 32) | libc::off64_t::from(len_high)
         } else {
             (libc::off64_t::from(len_high) << 32) | libc::off64_t::from(len_low)
@@ -138,7 +141,7 @@ pub(crate) fn sys_truncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
             // SAFETY:
             // 1. SysArg has one element.
             // 2. `/` is not a regular file -> EINVAL.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let fd = path_args
                 .0
                 .as_ref()
@@ -172,19 +175,11 @@ pub(crate) fn sys_truncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: FD-only call:
-    // 1. Assert valid fd before other arguments.
-    // 2. AT_FDCWD is an invalid fd argument.
-    if !is_valid_fd(req.data.args[0]) {
-        return request.fail_syscall(Errno::EBADF);
-    }
-
     // SAFETY: If second argument is negative, return EINVAL.
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
-    #[allow(clippy::cast_lossless)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         libc::off_t::from(req.data.args[1] as i32)
     } else {
@@ -195,6 +190,13 @@ pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EINVAL);
     }
 
+    // SAFETY: FD-only call:
+    // 1. Assert valid fd before other arguments.
+    // 2. AT_FDCWD is an invalid fd argument.
+    if !is_valid_fd(req.data.args[0]) {
+        return request.fail_syscall(Errno::EBADF);
+    }
+
     let argv = &[SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -206,7 +208,7 @@ pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. SysArg.path is None asserting dir is Some.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args
             .0
             .as_ref()
@@ -230,31 +232,27 @@ pub(crate) fn sys_ftruncate(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: FD-only call:
-    // 1. Assert valid fd before other arguments.
-    // 2. AT_FDCWD is an invalid fd argument.
-    if !is_valid_fd(req.data.args[0]) {
-        return request.fail_syscall(Errno::EBADF);
-    }
-
     // SAFETY: If second argument is negative, return EINVAL.
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     // truncate64 needs argument translation on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         // Align arg_no to the next even number for specific 32-bit architectures.
         let arg_no = match req.data.arch {
-            ScmpArch::Arm | ScmpArch::Ppc => 2,
+            ScmpArch::Arm | ScmpArch::Mipsel | ScmpArch::Ppc => 2,
             _ => 1,
         };
 
         let len_low = req.data.args[arg_no] as u32;
         let len_high = req.data.args[arg_no + 1] as u32;
 
-        if scmp_big_endian(req.data.arch) {
+        if req.data.arch == ScmpArch::Ppc && cfg!(target_pointer_width = "32") {
+            // PowerPC32 passes high word first, then low word.
+            (libc::off64_t::from(len_high) << 32) | libc::off64_t::from(len_low)
+        } else if scmp_big_endian(req.data.arch) {
             (libc::off64_t::from(len_low) << 32) | libc::off64_t::from(len_high)
         } else {
             (libc::off64_t::from(len_high) << 32) | libc::off64_t::from(len_low)
@@ -267,6 +265,13 @@ pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EINVAL);
     }
 
+    // SAFETY: FD-only call:
+    // 1. Assert valid fd before other arguments.
+    // 2. AT_FDCWD is an invalid fd argument.
+    if !is_valid_fd(req.data.args[0]) {
+        return request.fail_syscall(Errno::EBADF);
+    }
+
     let argv = &[SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -282,7 +287,7 @@ pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
             // SAFETY:
             // 1. SysArg has one element.
             // 2. SysArg.path is None asserting dir is Some.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
             // SAFETY: Record blocking call so it can get invalidated.
@@ -303,37 +308,20 @@ pub(crate) fn sys_ftruncate64(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: FD-only call:
-    // 1. Assert valid fd before other arguments.
-    // 2. AT_FDCWD is an invalid fd argument.
-    if !is_valid_fd(req.data.args[0]) {
-        return request.fail_syscall(Errno::EBADF);
-    }
-
-    // SAFETY: Reject undefined mode.
-    let mode = match req.data.args[1].try_into() {
-        Ok(mode) => mode,
-        Err(_) => return request.fail_syscall(Errno::EINVAL),
-    };
-
-    // SAFETY: Reject invalid mode.
-    let mode = match FallocateFlags::from_bits(mode) {
-        Some(mode) => mode,
-        None => return request.fail_syscall(Errno::EINVAL),
-    };
-
     // SAFETY: EINVAL offset was less than 0, or len was less than or equal to 0.
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     // fallocate needs argument translation for offset argument on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let off = if is32 {
         let len_low = req.data.args[2] as u32;
         let len_high = req.data.args[3] as u32;
 
-        if scmp_big_endian(req.data.arch) {
+        if req.data.arch == ScmpArch::Ppc && cfg!(target_pointer_width = "32") {
+            // PowerPC32 passes high word first, then low word.
+            (libc::off64_t::from(len_high) << 32) | libc::off64_t::from(len_low)
+        } else if scmp_big_endian(req.data.arch) {
             (libc::off64_t::from(len_low) << 32) | libc::off64_t::from(len_high)
         } else {
             (libc::off64_t::from(len_high) << 32) | libc::off64_t::from(len_low)
@@ -347,9 +335,8 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
     }
 
     // fallocate needs argument translation for length argument on some architectures.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_wrap)]
     let len = if is32 {
         let len_low = req.data.args[4] as u32;
         let len_high = req.data.args[5] as u32;
@@ -367,6 +354,19 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EINVAL);
     }
 
+    // SAFETY: Reject undefined/invalid mode.
+    let mode = match to_fallocate_flags(req.data.args[1]) {
+        Ok(mode) => mode,
+        Err(errno) => return request.fail_syscall(errno),
+    };
+
+    // SAFETY: FD-only call:
+    // 1. Assert valid fd before other arguments.
+    // 2. AT_FDCWD is an invalid fd argument.
+    if !is_valid_fd(req.data.args[0]) {
+        return request.fail_syscall(Errno::EBADF);
+    }
+
     let argv = &[SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -377,22 +377,69 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. SysArg.path is None asserting dir is Some.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
         // SAFETY: Record blocking call so it can get invalidated.
         request.cache.add_sys_block(req, false)?;
 
-        let result =
-                // SAFETY: nix does not have an interface for fallocate64.
-                Errno::result(unsafe { libc::fallocate64(fd.as_raw_fd(), mode.bits(), off, len) })
-                    .map(|_| request.return_syscall(0));
+        let result = fallocate64(fd, mode, off, len);
 
         // Remove invalidation record unless interrupted.
         request
             .cache
             .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
 
-        result
+        result.map(|_| request.return_syscall(0))
     })
 }
+
+fn to_fallocate_flags(arg: u64) -> Result<FallocateFlags, Errno> {
+    // TODO: Remove FALLOC_FL_WRITE_ZEROES logic once nix supports the flag.
+    // FALLOC_FL_MODE_MASK: FL_ALLOCATE_RANGE is encoded as 0, so it doesn't affect the mask.
+    const FALLOC_FL_WRITE_ZEROES: FallocateFlags = FallocateFlags::from_bits_retain(0x80);
+    const FALLOC_FL_MODE_MASK: c_int = FallocateFlags::FALLOC_FL_PUNCH_HOLE.bits()
+        | FallocateFlags::FALLOC_FL_COLLAPSE_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_ZERO_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_INSERT_RANGE.bits()
+        | FallocateFlags::FALLOC_FL_UNSHARE_RANGE.bits()
+        | FALLOC_FL_WRITE_ZEROES.bits();
+
+    let arg: c_int = arg.try_into().or(Err(Errno::EINVAL))?;
+    let all: c_int = FallocateFlags::all().bits() | FALLOC_FL_WRITE_ZEROES.bits();
+    if (arg & !all) != 0 {
+        return Err(Errno::EINVAL);
+    }
+
+    let mut flags = FallocateFlags::from_bits_truncate(arg);
+    if (arg & FALLOC_FL_WRITE_ZEROES.bits()) != 0 {
+        flags |= FALLOC_FL_WRITE_ZEROES;
+    }
+
+    // See Linux' vfs_fallocate() in fs/open.c for the validations.
+    if (arg & !(FALLOC_FL_MODE_MASK | FallocateFlags::FALLOC_FL_KEEP_SIZE.bits())) != 0 {
+        return Err(Errno::EOPNOTSUPP);
+    }
+
+    match arg & FALLOC_FL_MODE_MASK {
+        0 => { /* FALLOC_FL_ALLOCATE_RANGE */ }
+        x if x == FallocateFlags::FALLOC_FL_UNSHARE_RANGE.bits() => {}
+        x if x == FallocateFlags::FALLOC_FL_ZERO_RANGE.bits() => {}
+        x if x == FallocateFlags::FALLOC_FL_PUNCH_HOLE.bits() => {
+            if (arg & FallocateFlags::FALLOC_FL_KEEP_SIZE.bits()) == 0 {
+                return Err(Errno::EOPNOTSUPP);
+            }
+        }
+        x if x == FallocateFlags::FALLOC_FL_COLLAPSE_RANGE.bits()
+            || x == FallocateFlags::FALLOC_FL_INSERT_RANGE.bits()
+            || x == FALLOC_FL_WRITE_ZEROES.bits() =>
+        {
+            if (arg & FallocateFlags::FALLOC_FL_KEEP_SIZE.bits()) != 0 {
+                return Err(Errno::EOPNOTSUPP);
+            }
+        }
+        _ => return Err(Errno::EOPNOTSUPP),
+    }
+
+    Ok(flags)
+}
