@@ -8,36 +8,26 @@
 
 use std::{
     borrow::Cow,
-    ffi::CStr,
     mem::MaybeUninit,
     os::fd::{AsFd, AsRawFd},
 };
 
 use libseccomp::ScmpNotifResp;
-use memchr::memchr;
 use nix::{errno::Errno, fcntl::AtFlags, unistd::fchdir, NixPath};
 
 use crate::{
-    compat::{getxattrat, listxattrat, removexattrat, setxattrat, XattrArgs},
+    compat::{
+        getxattrat, listxattrat, removexattrat, setxattrat, XattrArgs, XATTR_LIST_MAX,
+        XATTR_SIZE_MAX,
+    },
     config::ROOT_FILE,
     fs::{denyxattr, filterxattr, FsFlags},
-    hook::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{syscall_path_handler, to_atflags},
     path::XPath,
+    req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
     sandbox::SandboxGuard,
 };
 
-/*
- * Constants from <linux/limits.h> not defined by libc yet.
- */
-
-// # chars in an extended attribute name.
-const XATTR_NAME_MAX: usize = 255;
-// size of an extended attribute value (64k).
-const XATTR_SIZE_MAX: usize = 1 << 16;
-// size of extended attribute namelist (64k).
-const XATTR_LIST_MAX: usize = 1 << 16;
-
 pub(crate) fn sys_getxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
     // SAFETY: We set WANT_BASE because fgetxattr requires a read-only
     // fd but we may not have access to open the file! Note, getxattr is
@@ -82,24 +72,19 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         // SAFETY:
         // 1. SysArg has one element.
         // 2. SysArg.path is None asserting dir is Some.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
         let req = request.scmpreq;
         let name = if req.data.args[1] != 0 {
-            let mut buf = Vec::new();
-            buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-            buf.resize(XATTR_NAME_MAX, 0);
-            request.read_mem(&mut buf, req.data.args[1])?;
-            Some(buf)
+            Some(request.read_xattr(req.data.args[1])?)
         } else {
             None
         };
-        let name = if let Some(ref name) = name {
-            to_name(name)?.as_ptr()
-        } else {
-            std::ptr::null()
-        };
+        let name = name
+            .as_ref()
+            .map(|cstr| cstr.as_ptr())
+            .unwrap_or(std::ptr::null());
 
         if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
             // SAFETY: Deny user.syd* extended attributes. name is either
@@ -126,7 +111,7 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             None => std::ptr::null_mut(),
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         // SAFETY: In libc we trust.
         let n = match Errno::result(unsafe {
             libc::fgetxattr(fd.as_raw_fd(), name, ptr.cast(), len)
@@ -145,7 +130,7 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             request.write_mem(&buf[..n], req.data.args[2])?;
         }
 
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         Ok(request.return_syscall(n as i64))
     })
 }
@@ -228,7 +213,7 @@ pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
     }];
     syscall_path_handler(request, "lsetxattr", argv, |path_args, request, sandbox| {
         // SAFETY: SysArg has one element.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let path = path_args.0.as_ref().unwrap();
 
         let base = if path.base.is_empty() {
@@ -240,19 +225,14 @@ pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         let req = request.scmpreq;
 
         let name = if req.data.args[1] != 0 {
-            let mut buf = Vec::new();
-            buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-            buf.resize(XATTR_NAME_MAX, 0);
-            request.read_mem(&mut buf, req.data.args[1])?;
-            Some(buf)
+            Some(request.read_xattr(req.data.args[1])?)
         } else {
             None
         };
-        let name = if let Some(ref name) = name {
-            to_name(name)?.as_ptr()
-        } else {
-            std::ptr::null()
-        };
+        let name = name
+            .as_ref()
+            .map(|cstr| cstr.as_ptr())
+            .unwrap_or(std::ptr::null());
 
         if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
             // SAFETY: Deny user.syd* extended attributes. name is either
@@ -272,12 +252,13 @@ pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             let mut buf = Vec::new();
             buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
             buf.resize(len, 0);
-            request.read_mem(&mut buf, req.data.args[2])?;
-            (Some(buf), len)
+            let n = request.read_mem(&mut buf, req.data.args[2])?;
+            buf.truncate(n);
+            (Some(buf), n)
         };
         let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
 
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         let flags = req.data.args[4] as libc::c_int;
 
         match &path.dir {
@@ -357,7 +338,7 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             // SAFETY:
             // 1. SysArg has one element.
             // 2. SysArg.path is None asserting dir is Some.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
 
             let req = request.scmpreq;
@@ -379,7 +360,7 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
                 .map_or(std::ptr::null_mut(), |b| b.as_mut_ptr())
                 as *mut libc::c_char;
 
-            #[allow(clippy::cast_sign_loss)]
+            #[expect(clippy::cast_sign_loss)]
             // SAFETY: In libc we trust.
             let n = match Errno::result(unsafe { libc::flistxattr(fd.as_raw_fd(), ptr, len) }) {
                 Ok(n) => n as usize,
@@ -407,7 +388,7 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
                 n
             };
 
-            #[allow(clippy::cast_possible_wrap)]
+            #[expect(clippy::cast_possible_wrap)]
             Ok(request.return_syscall(n as i64))
         },
     )
@@ -537,7 +518,7 @@ pub(crate) fn sys_lremovexattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         argv,
         |path_args, request, sandbox| {
             // SAFETY: SysArg has one element.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let path = path_args.0.as_ref().unwrap();
 
             let base = if path.base.is_empty() {
@@ -549,19 +530,14 @@ pub(crate) fn sys_lremovexattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             let req = request.scmpreq;
 
             let name = if req.data.args[1] != 0 {
-                let mut buf = Vec::new();
-                buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-                buf.resize(XATTR_NAME_MAX, 0);
-                request.read_mem(&mut buf, req.data.args[1])?;
-                Some(buf)
+                Some(request.read_xattr(req.data.args[1])?)
             } else {
                 None
             };
-            let name = if let Some(ref name) = name {
-                to_name(name)?.as_ptr()
-            } else {
-                std::ptr::null()
-            };
+            let name = name
+                .as_ref()
+                .map(|cstr| cstr.as_ptr())
+                .unwrap_or(std::ptr::null());
 
             if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
                 // SAFETY: Deny user.syd* extended attributes.
@@ -641,7 +617,7 @@ fn syscall_getxattr_handler(
     let req = request.scmpreq;
 
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let base = if path.base.is_empty() {
@@ -651,19 +627,14 @@ fn syscall_getxattr_handler(
     };
 
     let name = if req.data.args[1] != 0 {
-        let mut buf = Vec::new();
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-        request.read_mem(&mut buf, req.data.args[1])?;
-        Some(buf)
+        Some(request.read_xattr(req.data.args[1])?)
     } else {
         None
     };
-    let name = if let Some(ref name) = name {
-        to_name(name)?.as_ptr()
-    } else {
-        std::ptr::null()
-    };
+    let name = name
+        .as_ref()
+        .map(|cstr| cstr.as_ptr())
+        .unwrap_or(std::ptr::null());
 
     if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
@@ -703,7 +674,7 @@ fn syscall_getxattr_handler(
         // SAFETY: We do not resolve symbolic links here!
         .with_nix_path(|cstr| unsafe { libc::lgetxattr(cstr.as_ptr(), name, ptr.cast(), len) })?;
 
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_sign_loss)]
     let n = match Errno::result(res) {
         Ok(n) => n as usize,
         Err(Errno::ERANGE) if len == XATTR_SIZE_MAX => {
@@ -719,7 +690,7 @@ fn syscall_getxattr_handler(
         request.write_mem(&buf[..n], req.data.args[2])?;
     }
 
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_wrap)]
     Ok(request.return_syscall(n as i64))
 }
 
@@ -732,7 +703,7 @@ fn syscall_getxattrat_handler(
     let req = request.scmpreq;
 
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let base = if path.base.is_empty() {
@@ -745,7 +716,7 @@ fn syscall_getxattrat_handler(
     let mut args = MaybeUninit::<XattrArgs>::uninit();
 
     // SAFETY: Ensure size of XattrArgs matches with user argument.
-    if req.data.args[5] != std::mem::size_of::<XattrArgs>() as u64 {
+    if req.data.args[5] != size_of::<XattrArgs>() as u64 {
         return Err(Errno::EINVAL);
     }
 
@@ -753,10 +724,7 @@ fn syscall_getxattrat_handler(
     // We don't read uninitialized memory, and after `read_mem` fills it,
     // we're good to assume it's valid.
     let buf = unsafe {
-        std::slice::from_raw_parts_mut(
-            args.as_mut_ptr().cast::<u8>(),
-            std::mem::size_of::<XattrArgs>(),
-        )
+        std::slice::from_raw_parts_mut(args.as_mut_ptr().cast::<u8>(), size_of::<XattrArgs>())
     };
 
     // Read the remote data structure.
@@ -771,19 +739,14 @@ fn syscall_getxattrat_handler(
     }
 
     let name = if req.data.args[3] != 0 {
-        let mut buf = Vec::new();
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-        request.read_mem(&mut buf, req.data.args[3])?;
-        Some(buf)
+        Some(request.read_xattr(req.data.args[3])?)
     } else {
         None
     };
-    let name = if let Some(ref name) = name {
-        to_name(name)?.as_ptr()
-    } else {
-        std::ptr::null()
-    };
+    let name = name
+        .as_ref()
+        .map(|cstr| cstr.as_ptr())
+        .unwrap_or(std::ptr::null());
 
     if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
@@ -811,7 +774,7 @@ fn syscall_getxattrat_handler(
         None => return Err(Errno::EBADF),
     };
 
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     let mut my_args = XattrArgs {
         value: match buf.as_mut() {
             Some(b) => b.as_mut_ptr() as *mut libc::c_void as u64,
@@ -840,7 +803,7 @@ fn syscall_getxattrat_handler(
         request.write_mem(&buf[..n], args.value)?;
     }
 
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_wrap)]
     Ok(request.return_syscall(n as i64))
 }
 
@@ -853,24 +816,19 @@ fn syscall_setxattr_handler(
     // SAFETY:
     // 1. SysArg has one element.
     // 2. `/` is not permitted -> EACCES.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = args.0.as_ref().unwrap().dir.as_ref().ok_or(Errno::EACCES)?;
 
     let req = request.scmpreq;
     let name = if req.data.args[1] != 0 {
-        let mut buf = Vec::new();
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-        request.read_mem(&mut buf, req.data.args[1])?;
-        Some(buf)
+        Some(request.read_xattr(req.data.args[1])?)
     } else {
         None
     };
-    let name = if let Some(ref name) = name {
-        to_name(name)?.as_ptr()
-    } else {
-        std::ptr::null()
-    };
+    let name = name
+        .as_ref()
+        .map(|cstr| cstr.as_ptr())
+        .unwrap_or(std::ptr::null());
 
     if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
@@ -890,12 +848,13 @@ fn syscall_setxattr_handler(
         let mut buf = Vec::new();
         buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
         buf.resize(len, 0);
-        request.read_mem(&mut buf, req.data.args[2])?;
-        (Some(buf), len)
+        let n = request.read_mem(&mut buf, req.data.args[2])?;
+        buf.truncate(n);
+        (Some(buf), n)
     };
     let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
 
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     let flags = req.data.args[4] as libc::c_int;
 
     // SAFETY: In libc we trust.
@@ -912,7 +871,7 @@ fn syscall_setxattrat_handler(
     let req = request.scmpreq;
 
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let base = if path.base.is_empty() {
@@ -925,7 +884,7 @@ fn syscall_setxattrat_handler(
     let mut args = MaybeUninit::<XattrArgs>::uninit();
 
     // SAFETY: Ensure size of XattrArgs matches with user argument.
-    if req.data.args[5] != std::mem::size_of::<XattrArgs>() as u64 {
+    if req.data.args[5] != size_of::<XattrArgs>() as u64 {
         return Err(Errno::EINVAL);
     }
 
@@ -933,10 +892,7 @@ fn syscall_setxattrat_handler(
     // We don't read uninitialized memory, and after `read_mem` fills it,
     // we're good to assume it's valid.
     let buf = unsafe {
-        std::slice::from_raw_parts_mut(
-            args.as_mut_ptr().cast::<u8>(),
-            std::mem::size_of::<XattrArgs>(),
-        )
+        std::slice::from_raw_parts_mut(args.as_mut_ptr().cast::<u8>(), size_of::<XattrArgs>())
     };
 
     // Read the remote data structure.
@@ -946,19 +902,14 @@ fn syscall_setxattrat_handler(
     let args = unsafe { args.assume_init() };
 
     let name = if req.data.args[3] != 0 {
-        let mut buf = Vec::new();
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-        request.read_mem(&mut buf, req.data.args[3])?;
-        Some(buf)
+        Some(request.read_xattr(req.data.args[3])?)
     } else {
         None
     };
-    let name = if let Some(ref name) = name {
-        to_name(name)?.as_ptr()
-    } else {
-        std::ptr::null()
-    };
+    let name = name
+        .as_ref()
+        .map(|cstr| cstr.as_ptr())
+        .unwrap_or(std::ptr::null());
 
     if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
@@ -978,8 +929,9 @@ fn syscall_setxattrat_handler(
         let mut buf = Vec::new();
         buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
         buf.resize(len, 0);
-        request.read_mem(&mut buf, args.value)?;
-        (Some(buf), len)
+        let n = request.read_mem(&mut buf, args.value)?;
+        buf.truncate(n);
+        (Some(buf), n)
     };
     let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
 
@@ -988,7 +940,7 @@ fn syscall_setxattrat_handler(
         None => return Err(Errno::EBADF),
     };
 
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     let my_args = XattrArgs {
         value: buf as u64,
         size: len as u32,
@@ -1011,7 +963,7 @@ fn syscall_listxattr_handler(
     let req = request.scmpreq;
 
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let base = if path.base.is_empty() {
@@ -1049,7 +1001,7 @@ fn syscall_listxattr_handler(
         // SAFETY: We do not resolve symbolic links here!
         .with_nix_path(|cstr| unsafe { libc::llistxattr(cstr.as_ptr(), ptr, len) })?;
 
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_sign_loss)]
     let mut n = match Errno::result(res) {
         Ok(n) => n as usize,
         Err(Errno::ERANGE) if len == XATTR_LIST_MAX => {
@@ -1074,7 +1026,7 @@ fn syscall_listxattr_handler(
         n = buf.len();
     }
 
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_wrap)]
     Ok(request.return_syscall(n as i64))
 }
 
@@ -1087,7 +1039,7 @@ fn syscall_listxattrat_handler(
     let req = request.scmpreq;
 
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let base = if path.base.is_empty() {
@@ -1145,7 +1097,7 @@ fn syscall_listxattrat_handler(
         n = buf.len();
     }
 
-    #[allow(clippy::cast_possible_wrap)]
+    #[expect(clippy::cast_possible_wrap)]
     Ok(request.return_syscall(n as i64))
 }
 
@@ -1158,25 +1110,20 @@ fn syscall_removexattr_handler(
     // SAFETY:
     // 1. SysArg has one element.
     // 2. `/` is not permitted -> EACCES.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = args.0.as_ref().unwrap().dir.as_ref().ok_or(Errno::EACCES)?;
 
     let req = request.scmpreq;
 
     let name = if req.data.args[1] != 0 {
-        let mut buf = Vec::new();
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-        request.read_mem(&mut buf, req.data.args[1])?;
-        Some(buf)
+        Some(request.read_xattr(req.data.args[1])?)
     } else {
         None
     };
-    let name = if let Some(ref name) = name {
-        to_name(name)?.as_ptr()
-    } else {
-        std::ptr::null()
-    };
+    let name = name
+        .as_ref()
+        .map(|cstr| cstr.as_ptr())
+        .unwrap_or(std::ptr::null());
 
     if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes.
@@ -1200,7 +1147,7 @@ fn syscall_removexattrat_handler(
     let req = request.scmpreq;
 
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let base = if path.base.is_empty() {
@@ -1210,19 +1157,14 @@ fn syscall_removexattrat_handler(
     };
 
     let name = if req.data.args[3] != 0 {
-        let mut buf = Vec::new();
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-        request.read_mem(&mut buf, req.data.args[3])?;
-        Some(buf)
+        Some(request.read_xattr(req.data.args[3])?)
     } else {
         None
     };
-    let name = if let Some(ref name) = name {
-        to_name(name)?.as_ptr()
-    } else {
-        std::ptr::null()
-    };
+    let name = name
+        .as_ref()
+        .map(|cstr| cstr.as_ptr())
+        .unwrap_or(std::ptr::null());
 
     if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes.
@@ -1258,28 +1200,3 @@ fn to_len_val(arg: u64, max: usize) -> Result<usize, Errno> {
         _ => Err(Errno::ERANGE),
     }
 }
-
-// Validate extended attribute name.
-#[inline]
-fn to_name(name: &[u8]) -> Result<&CStr, Errno> {
-    // Note, the ERANGE error return here is misleading:
-    // https://bugzilla.kernel.org/show_bug.cgi?id=220374
-    let name = CStr::from_bytes_until_nul(name).or(Err(Errno::ERANGE))?;
-
-    // Check for non-empty name.
-    let buf = name.to_bytes();
-    let len = buf.len();
-    if len == 0 {
-        return Err(Errno::ERANGE);
-    }
-
-    // Check for qualified name in namespace.attribute form.
-    // EINVAL is expected here by sys-apps/attr's tests.
-    #[allow(clippy::arithmetic_side_effects)]
-    match memchr(b'.', buf) {
-        None => Err(Errno::EOPNOTSUPP),
-        Some(0) => Err(Errno::EINVAL),
-        Some(n) if n == len - 1 => Err(Errno::EINVAL),
-        _ => Ok(name),
-    }
-}
