@@ -206,7 +206,7 @@ impl PrivateAccess for AccessFs {
 }
 
 impl fmt::Display for AccessFs {
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         // Collect all flag-names in the same order used
         // when building `access` for the Syd sandbox.
@@ -264,7 +264,7 @@ impl fmt::Display for AccessFs {
 }
 
 impl Serialize for AccessFs {
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -352,7 +352,7 @@ where
 ///     Ok(PathBeneath::new(PathFd::new("/home")?, AccessFs::ReadDir))
 /// }
 /// ```
-#[cfg_attr(test, derive(Debug))]
+#[derive(Debug)]
 pub struct PathBeneath<F> {
     attr: uapi::landlock_path_beneath_attr,
     // Ties the lifetime of a file descriptor to this object.
@@ -557,7 +557,7 @@ fn path_beneath_compatibility() {
     path.set_compatibility(CompatLevel::HardRequirement);
 }
 
-// It is useful for documentation generation to explicitely implement Rule for every types, instead
+// It is useful for documentation generation to explicitly implement Rule for every types, instead
 // of doing it generically.
 impl<F> Rule<AccessFs> for PathBeneath<F> where F: AsFd {}
 
@@ -628,7 +628,7 @@ fn path_beneath_check_consistency() {
 ///     Ok(PathBeneath::new(fd, access))
 /// }
 /// ```
-#[cfg_attr(test, derive(Debug))]
+#[derive(Debug)]
 pub struct PathFd {
     pub fd: OwnedFd,
 }
