@@ -124,7 +124,7 @@ pub(crate) mod workers;
 /// Utilities for caching
 pub mod cache;
 /// Compatibility code for different libcs
-#[allow(missing_docs)]
+#[expect(missing_docs)]
 pub mod compat;
 /// Static configuration, edit & recompile!
 pub mod config;
@@ -150,6 +150,8 @@ pub mod landlock_policy;
 pub mod log;
 /// magic symlink utilities
 pub mod magic;
+/// Interface to Open File Description locks
+pub mod ofd;
 /// /proc and syd(2) nom parsers
 pub mod parsers;
 /// Path handling for UNIX
@@ -158,6 +160,10 @@ pub mod path;
 pub mod proc;
 /// ptrace(2) utilities
 pub mod ptrace;
+/// seccomp(2) notify request handling
+pub mod req;
+/// Utilities to handle restarting syscalls
+pub mod retry;
 /// OS Random Number Generator (RNG) interface
 pub mod rng;
 /// Sandbox configuration
@@ -165,7 +171,7 @@ pub mod sandbox;
 /// Execute program as sealed anonymous file
 pub mod seal;
 /// SealBox<T> for type-safe sealing/protecting
-#[allow(clippy::disallowed_types)]
+#[expect(clippy::disallowed_types)]
 pub mod sealbox;
 /// Portable sigset that can handle reserved signals
 pub mod sigset;
@@ -180,42 +186,29 @@ pub mod timer;
 /// Interface to uname(2)
 pub mod uts;
 /// Shell-style wildcard matching
-#[allow(clippy::arithmetic_side_effects)]
+#[expect(clippy::arithmetic_side_effects)]
 pub mod wildmatch;
 /// Interface to wordexp(3)
 pub mod wordexp;
 
 // Vendored crates:
 /// Interface to Linux capabilities
-#[allow(dead_code)]
-#[allow(missing_docs)]
-#[allow(clippy::arithmetic_side_effects)]
-#[allow(clippy::cast_possible_truncation)]
-#[allow(clippy::disallowed_types)]
-#[allow(clippy::missing_safety_doc)]
-#[allow(clippy::undocumented_unsafe_blocks)]
+#[expect(missing_docs)]
+#[expect(clippy::arithmetic_side_effects)]
+#[expect(clippy::undocumented_unsafe_blocks)]
 pub mod caps;
-// Used by Landlock crate.
-// lazy_static is a dev-only dependency.
-#[cfg(test)]
-#[macro_use]
-extern crate lazy_static;
 /// Interface to LandLock LSM
-#[allow(dead_code)]
-#[allow(missing_docs)]
-#[allow(unused_imports)]
-#[allow(clippy::as_underscore)]
-#[allow(clippy::borrow_as_ptr)]
-#[allow(clippy::cast_lossless)]
-#[allow(clippy::cast_possible_truncation)]
-#[allow(clippy::decimal_literal_representation)]
-#[allow(clippy::default_trait_access)]
-#[allow(clippy::disallowed_methods)]
-#[allow(clippy::disallowed_types)]
-#[allow(clippy::init_numbered_fields)]
-#[allow(clippy::missing_safety_doc)]
-#[allow(clippy::type_complexity)]
-#[allow(clippy::undocumented_unsafe_blocks)]
+#[expect(missing_docs)]
+#[expect(unused_imports)]
+#[expect(clippy::as_underscore)]
+#[expect(clippy::borrow_as_ptr)]
+#[expect(clippy::cast_lossless)]
+#[expect(clippy::cast_possible_truncation)]
+#[expect(clippy::decimal_literal_representation)]
+#[expect(clippy::default_trait_access)]
+#[expect(clippy::disallowed_methods)]
+#[expect(clippy::init_numbered_fields)]
+#[expect(clippy::undocumented_unsafe_blocks)]
 pub mod landlock;
 /// rusty_pool: Self growing / shrinking `ThreadPool` implementation
 pub(crate) mod pool;
@@ -262,6 +255,10 @@ use crate::{
 macro_rules! main {
     { $($body:tt)* } => {
         fn main() -> std::process::ExitCode {
+            // Silence mimalloc warnings.
+            #[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+            syd::config_mimalloc();
+
             match (|| -> syd::err::SydResult<std::process::ExitCode> { $($body)* })() {
                 Ok(code) => code,
                 Err(err) => {
@@ -288,7 +285,7 @@ macro_rules! main {
 /// Print Syd version information,
 /// and information about the system to
 /// standard output.
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     use crate::config::*;
 
@@ -351,7 +348,7 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     };
     printfln!("ASLR is {aslr}.")?;
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let bpf_jit = match std::fs::read_to_string("/proc/sys/net/core/bpf_jit_enable") {
         Ok(val) => match val.trim() {
             "0" => "disabled".to_string(),
@@ -569,15 +566,15 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
 
 /// Print Syd code name.
 pub fn syd_code_name() -> String {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let major = env!("CARGO_PKG_VERSION_MAJOR")
         .parse::<u64>()
         .expect("CARGO_PKG_VERSION_MAJOR");
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let minor = env!("CARGO_PKG_VERSION_MINOR")
         .parse::<u64>()
         .expect("CARGO_PKG_VERSION_MINOR");
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let patch = env!("CARGO_PKG_VERSION_PATCH")
         .parse::<u64>()
         .expect("CARGO_PKG_VERSION_PATCH");
@@ -828,8 +825,8 @@ pub(crate) fn set_cpu_priority_idle() -> Result<(), Errno> {
 }
 
 /// Simple human size formatter.
-#[allow(clippy::arithmetic_side_effects)]
-#[allow(clippy::cast_precision_loss)]
+#[expect(clippy::arithmetic_side_effects)]
+#[expect(clippy::cast_precision_loss)]
 pub fn human_size(bytes: usize) -> String {
     const SIZES: &[char] = &['B', 'K', 'M', 'G', 'T', 'P', 'E'];
     let factor = 1024usize;
@@ -845,11 +842,11 @@ pub fn human_size(bytes: usize) -> String {
     format!("{:.2}{}", size, SIZES[i])
 }
 
-#[allow(clippy::unnecessary_cast)]
+#[expect(clippy::unnecessary_cast)]
 const SIOCGIFINDEX: u64 = libc::SIOCGIFINDEX as u64;
-#[allow(clippy::unnecessary_cast)]
+#[expect(clippy::unnecessary_cast)]
 const SIOCGIFFLAGS: u64 = libc::SIOCGIFFLAGS as u64;
-#[allow(clippy::unnecessary_cast)]
+#[expect(clippy::unnecessary_cast)]
 const SIOCSIFFLAGS: u64 = libc::SIOCSIFFLAGS as u64;
 
 /// Functionally equivalent to "ifconfig lo up".
@@ -866,7 +863,7 @@ pub fn loopback_set_up() -> Result<i32, Errno> {
 
     // Prepare the interface request
     let mut ifreq = libc::ifreq {
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         ifr_name: [
             b'l' as libc::c_char,
             b'o' as libc::c_char,
@@ -934,7 +931,7 @@ pub fn loopback_set_up() -> Result<i32, Errno> {
     // Modify the flags to bring up the interface.
     //
     // SAFETY: We're accessing the field of a union here.
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     unsafe {
         ifreq.ifr_ifru.ifru_flags |= (libc::IFF_UP | libc::IFF_RUNNING) as libc::c_short
     };
@@ -1006,11 +1003,10 @@ pub fn loopback_set_bigtcp(ifindex: i32, max_size: u32) -> Result<(), Errno> {
 
     // SAFETY: Set up netlink header.
     let nl_hdr = &mut msg.hdr;
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
     {
-        nl_hdr.nlmsg_len =
-            (std::mem::size_of::<libc::nlmsghdr>() + std::mem::size_of::<ifinfomsg>()) as u32;
+        nl_hdr.nlmsg_len = (size_of::<libc::nlmsghdr>() + size_of::<ifinfomsg>()) as u32;
         nl_hdr.nlmsg_type = libc::RTM_NEWLINK;
         nl_hdr.nlmsg_flags = (libc::NLM_F_REQUEST | libc::NLM_F_ACK) as u16;
         nl_hdr.nlmsg_seq = 1;
@@ -1019,7 +1015,7 @@ pub fn loopback_set_bigtcp(ifindex: i32, max_size: u32) -> Result<(), Errno> {
 
     // SAFETY: Populate ifinfomsg.
     let info = &mut msg.info;
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     {
         info.family = libc::AF_UNSPEC as u8;
         info.index = ifindex;
@@ -1034,28 +1030,27 @@ pub fn loopback_set_bigtcp(ifindex: i32, max_size: u32) -> Result<(), Errno> {
         IFLA_GSO_MAX_SIZE,
     ] {
         // SAFETY: Set attribute header.
-        #[allow(clippy::cast_ptr_alignment)]
+        #[expect(clippy::cast_ptr_alignment)]
         let attr_ptr = unsafe { msg.attrs.as_mut_ptr().add(offset) as *mut nlattr };
 
         // SAFETY: Write attribute metadata.
-        #[allow(clippy::arithmetic_side_effects)]
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::arithmetic_side_effects)]
+        #[expect(clippy::cast_possible_truncation)]
         unsafe {
             (*attr_ptr).nla_type = kind;
-            (*attr_ptr).nla_len =
-                (std::mem::size_of::<nlattr>() + std::mem::size_of::<u32>()) as u16;
+            (*attr_ptr).nla_len = (size_of::<nlattr>() + size_of::<u32>()) as u16;
         }
 
         // SAFETY: Write u32 payload.
-        #[allow(clippy::cast_ptr_alignment)]
+        #[expect(clippy::cast_ptr_alignment)]
         unsafe {
-            let ptr = (attr_ptr as *mut u8).add(std::mem::size_of::<nlattr>()) as *mut u32;
+            let ptr = (attr_ptr as *mut u8).add(size_of::<nlattr>()) as *mut u32;
             *ptr = max_size;
         }
 
-        #[allow(clippy::arithmetic_side_effects)]
-        #[allow(clippy::cast_lossless)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::arithmetic_side_effects)]
+        #[expect(clippy::cast_lossless)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // SAFETY: NLA_ALIGN ensures proper alignment for netlink attributes
             // as required by Linux kernel ABI.
@@ -1063,8 +1058,8 @@ pub fn loopback_set_bigtcp(ifindex: i32, max_size: u32) -> Result<(), Errno> {
         }
     }
 
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
     {
         msg.hdr.nlmsg_len += offset as u32;
     }
@@ -1086,37 +1081,34 @@ pub fn loopback_set_bigtcp(ifindex: i32, max_size: u32) -> Result<(), Errno> {
 
     // Check response messages for error.
     let mut offset = 0;
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_ptr_alignment)]
-    while offset + std::mem::size_of::<libc::nlmsghdr>() <= buf.len() {
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_ptr_alignment)]
+    while offset + size_of::<libc::nlmsghdr>() <= buf.len() {
         // SAFETY: We're reading a netlink message header from validated bounds.
         let hdr = unsafe { &*(buf.as_ptr().add(offset) as *const libc::nlmsghdr) };
 
         let len = hdr.nlmsg_len as usize;
-        if len < std::mem::size_of::<libc::nlmsghdr>() || offset + len > buf.len() {
+        if len < size_of::<libc::nlmsghdr>() || offset + len > buf.len() {
             return Err(Errno::EINVAL);
         }
 
         // Check for error message.
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         if hdr.nlmsg_type == libc::NLMSG_ERROR as libc::c_ushort
-            && len >= std::mem::size_of::<libc::nlmsghdr>() + std::mem::size_of::<libc::nlmsgerr>()
+            && len >= size_of::<libc::nlmsghdr>() + size_of::<libc::nlmsgerr>()
         {
             // SAFETY: Enough data to safely parse nlmsgerr.
             let err = unsafe {
-                &*(buf
-                    .as_ptr()
-                    .add(offset + std::mem::size_of::<libc::nlmsghdr>())
-                    as *const libc::nlmsgerr)
+                &*(buf.as_ptr().add(offset + size_of::<libc::nlmsghdr>()) as *const libc::nlmsgerr)
             };
             if err.error != 0 {
                 return Err(Errno::from_raw(-err.error));
             }
         }
 
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // SAFETY: nlmsg_len is kernel-aligned; advance to next message.
             offset += unsafe { libc::NLA_ALIGN(len as i32) as usize };
@@ -1167,12 +1159,45 @@ pub fn set_sigpipe_dfl() -> Result<(), Errno> {
     unsafe { signal(Signal::SIGPIPE, SigHandler::SigDfl) }.map(drop)
 }
 
-#[allow(dead_code)]
+/// Configure mimalloc to silence stderr.
+///
+/// This function is NOT thread-safe, call it early at init.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[allow(non_upper_case_globals)]
+pub fn config_mimalloc() {
+    // Extended feature is not usable (breaks on loongarch64),
+    // therefore we define a subset of mimalloc API here.
+    #[allow(non_camel_case_types)]
+    type mi_option_t = libc::c_int;
+    #[allow(non_camel_case_types)]
+    type mi_output_fun =
+        Option<unsafe extern "C" fn(msg: *const libc::c_char, arg: *mut libc::c_void)>;
+
+    #[link(name = "mimalloc")]
+    extern "C" {
+        fn mi_option_set_enabled(option: mi_option_t, enable: bool);
+        fn mi_register_output(out: mi_output_fun, arg: *mut libc::c_void);
+    }
+    extern "C" fn syd_noop_output(_msg: *const libc::c_char, _arg: *mut libc::c_void) {}
+
+    const mi_option_show_errors: mi_option_t = 0;
+    const mi_option_show_stats: mi_option_t = 1;
+    const mi_option_verbose: mi_option_t = 2;
+
+    // SAFETY: In mimalloc we trust.
+    unsafe {
+        mi_option_set_enabled(mi_option_show_errors, false);
+        mi_option_set_enabled(mi_option_show_stats, false);
+        mi_option_set_enabled(mi_option_verbose, false);
+        mi_register_output(Some(syd_noop_output), std::ptr::null_mut());
+    }
+}
+
 #[inline]
 #[cold]
 fn cold() {}
 
-#[allow(dead_code)]
+#[expect(dead_code)]
 #[inline]
 pub(crate) fn likely(b: bool) -> bool {
     if !b {
@@ -1181,7 +1206,7 @@ pub(crate) fn likely(b: bool) -> bool {
     b
 }
 
-#[allow(dead_code)]
+#[expect(dead_code)]
 #[inline]
 pub(crate) fn unlikely(b: bool) -> bool {
     if b {
@@ -1217,7 +1242,7 @@ macro_rules! T {
 
 #[cfg(feature = "prof")]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn start_cpu_profile(name: &str) {
     gperftools::profiler::PROFILER
         .lock()
@@ -1228,12 +1253,12 @@ pub(crate) fn start_cpu_profile(name: &str) {
 
 #[cfg(not(feature = "prof"))]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn start_cpu_profile(_name: &str) {}
 
 #[cfg(feature = "prof")]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn stop_cpu_profile() {
     gperftools::profiler::PROFILER
         .lock()
@@ -1244,12 +1269,12 @@ pub(crate) fn stop_cpu_profile() {
 
 #[cfg(not(feature = "prof"))]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn stop_cpu_profile() {}
 
 #[cfg(feature = "prof")]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn start_mem_profile(name: &str) {
     gperftools::heap_profiler::HEAP_PROFILER
         .lock()
@@ -1260,12 +1285,12 @@ pub(crate) fn start_mem_profile(name: &str) {
 
 #[cfg(not(feature = "prof"))]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn start_mem_profile(_name: &str) {}
 
 #[cfg(feature = "prof")]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn dump_mem_profile(name: &str) {
     gperftools::heap_profiler::HEAP_PROFILER
         .lock()
@@ -1276,12 +1301,12 @@ pub(crate) fn dump_mem_profile(name: &str) {
 
 #[cfg(not(feature = "prof"))]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn dump_mem_profile(_name: &str) {}
 
 #[cfg(feature = "prof")]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn stop_mem_profile() {
     gperftools::heap_profiler::HEAP_PROFILER
         .lock()
@@ -1292,5 +1317,5 @@ pub(crate) fn stop_mem_profile() {
 
 #[cfg(not(feature = "prof"))]
 #[inline(always)]
-#[allow(dead_code)]
+#[expect(dead_code)]
 pub(crate) fn stop_mem_profile() {}
