@@ -22,6 +22,7 @@ use nom::{
     bytes::complete::{tag, tag_no_case, take_while1},
     character::complete::{char, digit1, one_of},
     combinator::{all_consuming, map, opt, recognize},
+    error::{Error, ErrorKind},
     multi::separated_list1,
     sequence::preceded,
     Finish, IResult, Parser,
@@ -206,7 +207,7 @@ pub(crate) fn parse_bind_cmd(command: &str) -> Result<BindCmd, Errno> {
     fn inner(input: &str) -> IResult<&str, (char, &str, &str, Option<&str>)> {
         // Sequence: "bind", one of '+','-','^', <src> (no ':'), ":", <dst> (no ':'), optional ":" + <opt>
         (
-            nom::bytes::complete::tag("bind"),
+            tag("bind"),
             alt((char('+'), char('-'), char('^'))),
             take_while1(|c| c != ':'), // src
             char(':'),                 // consume colon
@@ -336,10 +337,7 @@ pub(crate) fn parse_force_cmd(input: &str) -> Result<ForceCmd, Errno> {
             match Action::from_str(act) {
                 Ok(act) => Some(act),
                 Err(_) => {
-                    return Err(nom::Err::Error(nom::error::Error::new(
-                        input,
-                        nom::error::ErrorKind::Fail,
-                    )));
+                    return Err(nom::Err::Error(Error::new(input, ErrorKind::Fail)));
                 }
             }
         } else {
@@ -582,9 +580,9 @@ pub(crate) fn parse_scmp_cmd(input: &str) -> Result<ScmpCmd, Errno> {
                 return Err(Errno::EINVAL);
             }
 
-            // IP address arguments are only valid for the `inet` set.
-            let maybe_addr = filter.intersects(Capability::CAP_INET)
-                && filter.difference(Capability::CAP_INET).is_empty();
+            // IP address arguments are only valid for the `net` set.
+            let maybe_addr = filter.intersects(Capability::CAP_NET)
+                && filter.difference(Capability::CAP_NET).is_empty();
 
             // Pattern resolution.
             let pat = if maybe_addr {
@@ -687,10 +685,7 @@ fn netlink_parser(input: &str) -> IResult<&str, Vec<String>> {
     // Validate each.
     for &fam in &raw_list {
         if NETLINK_FAMILIES.binary_search(&fam).is_err() {
-            return Err(nom::Err::Error(nom::error::Error::new(
-                input,
-                nom::error::ErrorKind::Fail,
-            )));
+            return Err(nom::Err::Error(Error::new(input, ErrorKind::Fail)));
         }
     }
 
@@ -1619,7 +1614,7 @@ mod tests {
                 action: Action::Stop,
                 filter: Capability::CAP_NET,
                 op: '^',
-                pat: ScmpPattern::Path("1.2.3.4!80".into()),
+                pat: ScmpPattern::Addr("1.2.3.4!80".into()),
             }
         );
     }
