@@ -870,6 +870,11 @@ impl XPath {
         self.is_equal(b"/proc/version")
     }
 
+    /// Check if path points to `/proc/sys/kernel/osrelease`.
+    pub fn is_proc_osrelease(&self) -> bool {
+        self.is_equal(b"/proc/sys/kernel/osrelease")
+    }
+
     /// Check if path points to machine-id(5), /etc/hostid or /var/adm/hostid.
     pub fn is_machine_id(&self) -> bool {
         for f in [
