@@ -36,7 +36,6 @@ use std::{
         unix::ffi::OsStrExt,
     },
     ptr::NonNull,
-    sync::LazyLock,
 };
 
 use bitflags::bitflags;
@@ -44,7 +43,7 @@ use btoi::{btoi, btoi_radix};
 use memchr::{arch::all::is_prefix, memrchr};
 use nix::{
     errno::Errno,
-    fcntl::{readlinkat, OFlag, ResolveFlag},
+    fcntl::{OFlag, ResolveFlag},
     libc::pid_t,
     sys::{
         stat::Mode,
@@ -69,16 +68,20 @@ use crate::{
     cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
     elf::ElfType,
     err::{err2no, proc_error_to_errno},
-    fs::{is_open_fd, parse_fd, read_all, safe_open, write_all, AT_BADFD},
+    fs::{is_open_fd, parse_fd, read_all, readlinkat, safe_open, write_all, AT_BADFD},
     hash::{SydHashMap, SydHashSet, SydIndexSet},
     parsers::{
         map_result,
-        proc::{parse_max_open_files, parse_stat, parse_statm, parse_status},
+        proc::{
+            parse_max_open_files, parse_stat, parse_statm, parse_status, parse_status_tgid,
+            parse_status_umask,
+        },
         read_to_end,
     },
     path::{XPathBuf, PATH_MAX},
     retry::retry_on_eintr,
     rng::randint,
+    sandbox::Sandbox,
     sigset::SydSigSet,
     XPath,
 };
@@ -147,8 +150,6 @@ impl From<MMPermissions> for ProcmapQueryFlags {
     }
 }
 
-#[repr(C)]
-#[derive(Debug, Clone, Copy)]
 /// Input/output argument structure passed into the _ioctl_(2) call. It can
 /// be used to query a set of VMAs (Virtual Memory Areas) of a process.
 ///
@@ -170,6 +171,8 @@ impl From<MMPermissions> for ProcmapQueryFlags {
 /// _proc_pid_maps_(5), but `procmap_query` provides more querying flexibility.
 /// There are no consistency guarantees between subsequent _ioctl_(2) calls,
 /// but data returned for a matched VMA is self-consistent.
+#[repr(C)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
 pub struct ProcmapQuery {
     /// Query struct size, for backwards/forward compatibility.
     pub size: u64,
@@ -238,7 +241,7 @@ pub struct ProcmapQuery {
 impl Default for ProcmapQuery {
     fn default() -> Self {
         Self {
-            size: std::mem::size_of::<Self>() as u64,
+            size: size_of::<Self>() as u64,
             query_flags: 0,
             query_addr: 0,
             vma_start: 0,
@@ -324,7 +327,7 @@ pub fn procmap_query<Fd: AsFd>(
 /// All values are in units of pages.
 ///
 /// See `man 5 proc` and `Linux/fs/proc/array.c`.
-#[derive(Debug, Default, PartialEq, Eq, Hash)]
+#[derive(Copy, Clone, Debug, Default, PartialEq, Eq, Hash)]
 pub struct Statm {
     /// Total virtual memory size.
     pub size: u64,
@@ -333,10 +336,12 @@ pub struct Statm {
 /// Process status information.
 ///
 /// See `man 5 proc` and `Linux/fs/proc/array.c`.
-#[derive(Debug, Default, PartialEq, Eq, Hash)]
+#[derive(Copy, Clone, Debug, Default, PartialEq, Eq, Hash)]
 pub struct Stat {
     /// Number of threads in this process (since Linux 2.6).
     pub num_threads: u64,
+    /// The address above which program heap can be expanded with brk(2).
+    pub startbrk: u64,
     /// The address of the start (i.e., bottom) of the stack.
     pub startstack: u64,
     /// The controlling terminal of the process. (The minor device
@@ -348,14 +353,12 @@ pub struct Stat {
 /// Process status information.
 ///
 /// See `man 5 proc` and `Linux/fs/proc/array.c`.
-#[derive(Default, Debug, PartialEq, Eq, Hash)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
 pub struct Status {
-    /// Filename of the executable.
-    pub command: XPathBuf,
     /// File mode creation mask (since Linux 4.7).
-    pub umask: libc::mode_t,
+    pub umask: Mode,
     /// Process ID (i.e., Thread Group ID).
-    pub pid: libc::pid_t,
+    pub pid: Pid,
     /// Number of signals pending for the thread (see pthreads(7)).
     pub sig_pending_thread: SydSigSet,
     /// Number of signals pending for the process (see signal(7)).
@@ -368,6 +371,22 @@ pub struct Status {
     pub sig_caught: SydSigSet,
 }
 
+// Default is used by parsers.
+// Pid has no default therefore this is needed.
+impl Default for Status {
+    fn default() -> Self {
+        Self {
+            umask: Mode::empty(),
+            pid: Pid::from_raw(0),
+            sig_pending_thread: SydSigSet::default(),
+            sig_pending_process: SydSigSet::default(),
+            sig_blocked: SydSigSet::default(),
+            sig_ignored: SydSigSet::default(),
+            sig_caught: SydSigSet::default(),
+        }
+    }
+}
+
 /// Safe `OFlags` to use when opening proc(5) files.
 pub const OFLAG_PROC: OFlag =
     OFlag::from_bits_retain(libc::O_RDONLY | libc::O_CLOEXEC | libc::O_NOFOLLOW | libc::O_NOCTTY);
@@ -505,14 +524,28 @@ pub fn proc_status_read<Fd: AsFd>(fd: Fd) -> Result<Status, Errno> {
 
 /// Retrieves the thread group ID (TGID) for the specified thread ID (TID).
 pub fn proc_tgid(tid: Pid) -> Result<Pid, Errno> {
-    proc_status(tid).map(|stat| stat.pid).map(Pid::from_raw)
+    // This function is a hot-path, so we provide a specialized impl.
+    // proc_status(tid).map(|stat| stat.pid)
+    let fd = proc_status_open(tid)?;
+
+    // Read up to 192 bytes.
+    let mut data = [0u8; 192];
+    let nread = read_all(fd, &mut data)?;
+
+    map_result(parse_status_tgid(&data[..nread]))
 }
 
 /// Get the umask of the given `Pid`.
 pub fn proc_umask(pid: Pid) -> Result<Mode, Errno> {
-    proc_status(pid)
-        .map(|stat| stat.umask)
-        .map(Mode::from_bits_truncate)
+    // This function is a hot-path, so we provide a specialized impl.
+    // proc_status(pid).map(|stat| stat.umask)
+    let fd = proc_status_open(pid)?;
+
+    // Read up to 160 bytes.
+    let mut data = [0u8; 160];
+    let nread = read_all(fd, &mut data)?;
+
+    map_result(parse_status_umask(&data[..nread]))
 }
 
 /// Locate stack and return the stack memory address range.
@@ -619,9 +652,7 @@ pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
 pub fn proc_cwd(pid: Pid) -> Result<XPathBuf, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"cwd");
-    readlinkat(PROC_FILE(), &path)
-        .map(XPathBuf::from)
-        .map_err(proc_errno)
+    readlinkat(PROC_FILE(), &path).map_err(proc_errno)
 }
 
 /// Retrieves the command name (comm) of the specified process ID (PID)
@@ -1458,31 +1489,7 @@ pub fn proc_unix_inodes(pid: Pid) -> Result<SydHashSet<u64>, Errno> {
 pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     // This function requires CONFIG_CROSS_MEMORY_ATTACH,
     // unless Syd is explicitly configured to fallback.
-    static FORCE_PROC: LazyLock<bool> =
-        LazyLock::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
-    static PROC_MEM_F: LazyLock<bool> =
-        LazyLock::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
-    proc_set_at_secure2(
-        pid,
-        elf_type,
-        *FORCE_PROC,
-        *PROC_MEM_F,
-        *HAVE_CROSS_MEMORY_ATTACH,
-    )
-}
-
-/// Sets the AT_SECURE value to 1 in the auxiliary vector of the
-/// specified process.
-pub fn proc_set_at_secure2(
-    pid: Pid,
-    elf_type: ElfType,
-    force_proc: bool,
-    proc_mem_f: bool,
-    have_cma: bool,
-) -> Result<(), Errno> {
-    // This function requires CONFIG_CROSS_MEMORY_ATTACH,
-    // unless Syd is explicitly configured to fallback.
-    let mut proc_mem = if force_proc || (proc_mem_f && !have_cma) {
+    let mut proc_mem = if Sandbox::use_proc_pid_mem() {
         let mut path = XPathBuf::from_pid(pid);
         path.push(b"mem");
 
@@ -1768,8 +1775,9 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         RFLAG_PROC,
     )?;
 
-    let mut dot = 0u8;
     let mut res = vec![];
+    let mut seen_dot = false;
+    let mut seen_dotdot = false;
     loop {
         let mut entries = match getdents64(&dir, DIRENT_BUF_SIZE) {
             Ok(entries) => entries,
@@ -1778,13 +1786,17 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         };
 
         for entry in &mut entries {
-            #[expect(clippy::arithmetic_side_effects)]
-            if dot < 2 && entry.is_dot() {
-                dot += 1;
+            if !seen_dot && entry.is_dot() {
+                seen_dot = true;
                 continue;
             }
-            let fd = parse_fd(XPath::from_bytes(entry.name_bytes()))?;
-            let target = readlinkat(&dir, entry.name_bytes()).map(XPathBuf::from)?;
+            if !seen_dotdot && entry.is_dotdot() {
+                seen_dotdot = true;
+                continue;
+            }
+            let entry = XPath::from_bytes(entry.name_bytes());
+            let fd = parse_fd(entry)?;
+            let target = readlinkat(&dir, entry)?;
 
             res.push((fd, target));
         }
@@ -2004,6 +2016,9 @@ mod tests {
             return;
         }
 
+        let result = proc_tgid(Pid::from_raw(i32::MAX));
+        assert!(result.is_err(), "{result:?}");
+
         let result = proc_umask(Pid::from_raw(i32::MAX));
         assert!(result.is_err(), "{result:?}");
     }
@@ -2047,7 +2062,7 @@ mod tests {
     fn test_procmap_query_default_struct() {
         // Default values should be correct and size set properly.
         let q = ProcmapQuery::default();
-        assert_eq!(q.size as usize, std::mem::size_of::<ProcmapQuery>());
+        assert_eq!(q.size as usize, size_of::<ProcmapQuery>());
         assert_eq!(q.query_flags, 0);
         assert_eq!(q.query_addr, 0);
         assert_eq!(q.vma_start, 0);
@@ -2319,7 +2334,7 @@ mod tests {
             None,
         )
         .unwrap();
-        assert_eq!(q.size as usize, std::mem::size_of::<ProcmapQuery>());
+        assert_eq!(q.size as usize, size_of::<ProcmapQuery>());
     }
 
     #[test]
