@@ -19,7 +19,7 @@ use std::{
         fd::{AsFd, AsRawFd, OwnedFd, RawFd},
         unix::ffi::OsStrExt,
     },
-    sync::{Arc, LazyLock, RwLock},
+    sync::{Arc, RwLock},
 };
 
 use bitflags::bitflags;
@@ -42,10 +42,7 @@ use serde::{ser::SerializeMap, Serialize};
 
 use crate::{
     compat::{fstatx, timespec_tv_nsec_t, TimeSpec32, TimeSpec64, STATX_INO, XATTR_NAME_MAX},
-    config::{
-        ENV_NO_CROSS_MEMORY_ATTACH, ENV_PROC_PID_MEM_FALLBACK, HAVE_CROSS_MEMORY_ATTACH,
-        MMAP_MIN_ADDR, PROC_FILE,
-    },
+    config::{MMAP_MIN_ADDR, PAGE_SIZE, PROC_FILE},
     confine::{op2name, scmp_arch_bits, ScmpNotifReq, SydArch, Sydcall, EIDRM},
     error,
     fs::{
@@ -55,7 +52,7 @@ use crate::{
     },
     path::{XPath, XPathBuf, PATH_MAX, PATH_MIN},
     proc::{
-        proc_auxv, proc_maps, proc_rand_fd, proc_stack_pointer, proc_status, proc_tgid,
+        proc_auxv, proc_comm, proc_maps, proc_rand_fd, proc_stack_pointer, proc_status, proc_tgid,
         proc_unix_inodes,
     },
     sandbox::{Action, Flags, Sandbox, SandboxGuard},
@@ -156,6 +153,10 @@ impl SysArg {
             fsflags |= FsFlags::RESOLVE_BENEATH;
         }
 
+        if rflags.contains(ResolveFlag::RESOLVE_IN_ROOT) {
+            fsflags |= FsFlags::RESOLVE_IN_ROOT;
+        }
+
         if rflags.contains(ResolveFlag::RESOLVE_NO_SYMLINKS) {
             fsflags |= FsFlags::NO_RESOLVE_PATH;
         }
@@ -422,13 +423,7 @@ impl RemoteProcess {
         local_buffer: &mut [u8],
         remote_addr: u64,
     ) -> Result<usize, Errno> {
-        static FORCE_PROC: LazyLock<bool> =
-            LazyLock::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
-        // SAFETY: Falling back to proc_pid_mem(5) is insecure,
-        // and is no longer the default as of 3.32.6.
-        static PROC_MEM_F: LazyLock<bool> =
-            LazyLock::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
-        if *FORCE_PROC || (*PROC_MEM_F && !*HAVE_CROSS_MEMORY_ATTACH) {
+        if Sandbox::use_proc_pid_mem() {
             return self.read_mem_proc(local_buffer, remote_addr);
         }
 
@@ -496,13 +491,7 @@ impl RemoteProcess {
         local_buffer: &[u8],
         remote_addr: u64,
     ) -> Result<usize, Errno> {
-        static FORCE_PROC: LazyLock<bool> =
-            LazyLock::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
-        // SAFETY: Falling back to proc_pid_mem(5) is insecure,
-        // and is no longer the default as of 3.32.6.
-        static PROC_MEM_F: LazyLock<bool> =
-            LazyLock::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
-        if *FORCE_PROC || (*PROC_MEM_F && !*HAVE_CROSS_MEMORY_ATTACH) {
+        if Sandbox::use_proc_pid_mem() {
             return self.write_mem_proc(local_buffer, remote_addr);
         }
 
@@ -643,18 +632,18 @@ impl Serialize for UNotifyEventRequest {
         map.serialize_entry("args", &self.scmpreq.data.args)?;
         map.serialize_entry("arch", &SydArch(self.scmpreq.data.arch))?;
 
-        #[expect(clippy::cast_possible_wrap)]
-        let pid = Pid::from_raw(self.scmpreq.pid as libc::pid_t);
-
+        let pid = self.scmpreq.pid();
+        if let Ok(comm) = proc_comm(pid) {
+            map.serialize_entry("cmd", &comm)?;
+        }
         if let Ok(status) = proc_status(pid) {
-            map.serialize_entry("cmd", &status.command)?;
-            map.serialize_entry("tgid", &status.pid)?;
+            map.serialize_entry("tgid", &status.pid.as_raw())?;
             map.serialize_entry("sig_caught", &status.sig_caught)?;
             map.serialize_entry("sig_blocked", &status.sig_blocked)?;
             map.serialize_entry("sig_ignored", &status.sig_ignored)?;
             map.serialize_entry("sig_pending_thread", &status.sig_pending_thread)?;
             map.serialize_entry("sig_pending_process", &status.sig_pending_process)?;
-            map.serialize_entry("umask", &status.umask)?;
+            map.serialize_entry("umask", &status.umask.bits())?;
         }
 
         #[expect(clippy::unnecessary_cast)]
@@ -1033,25 +1022,38 @@ impl UNotifyEventRequest {
     /// Read the `libc::open_how` struct from process memory
     /// at the given address and size.
     pub(crate) fn remote_ohow(&self, addr: u64, size: u64) -> Result<libc::open_how, Errno> {
-        const OPEN_HOW_SIZE: usize = std::mem::size_of::<libc::open_how>();
-        const OPEN_HOW_ALIGN: usize = std::mem::align_of::<libc::open_how>();
+        const OPEN_HOW_SIZE_VER0: usize = 24;
+        const OPEN_HOW_SIZE_LATEST: usize = size_of::<libc::open_how>();
 
         // SAFETY: Validate size argument.
         let size = usize::try_from(size).or(Err(Errno::EINVAL))?;
-        match size.cmp(&OPEN_HOW_SIZE) {
-            Ordering::Equal => {}
-            Ordering::Less => return Err(Errno::EINVAL),
-            Ordering::Greater if size % OPEN_HOW_ALIGN == 0 => return Err(Errno::E2BIG),
-            Ordering::Greater => return Err(Errno::EFAULT),
-        };
+        if size < OPEN_HOW_SIZE_VER0 {
+            return Err(Errno::EINVAL);
+        }
+        if size as u64 > *PAGE_SIZE {
+            return Err(Errno::E2BIG);
+        }
 
         // SAFETY: Validate address argument.
         if addr < *MMAP_MIN_ADDR {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; OPEN_HOW_SIZE];
-        self.read_mem(&mut buf, addr)?;
+        // Allocate buffer.
+        // Size is already capped to page size.
+        let mut raw = Vec::new();
+        raw.try_reserve(size).or(Err(Errno::ENOMEM))?;
+        raw.resize(size, 0);
+        self.read_mem(&mut raw, addr)?;
+
+        // SAFETY: Verify trailing bytes are zero; otherwise E2BIG.
+        if raw.iter().skip(OPEN_HOW_SIZE_LATEST).any(|&b| b != 0) {
+            return Err(Errno::E2BIG);
+        }
+
+        let mut buf = [0u8; OPEN_HOW_SIZE_LATEST];
+        let len = buf.len().min(size);
+        buf[..len].copy_from_slice(&raw[..len]);
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of open_how in our Rust environment
@@ -1076,7 +1078,7 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; std::mem::size_of::<libc::utimbuf>()];
+        let mut buf = [0u8; size_of::<libc::utimbuf>()];
         self.read_mem(&mut buf, addr)?;
 
         // SAFETY: The following unsafe block assumes that:
@@ -1107,7 +1109,7 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; std::mem::size_of::<libc::timeval>() * 2];
+        let mut buf = [0u8; size_of::<libc::timeval>() * 2];
         self.read_mem(&mut buf, addr)?;
 
         // SAFETY: The following unsafe block assumes that:
@@ -1150,7 +1152,7 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; std::mem::size_of::<TimeSpec32>()];
+        let mut buf = [0u8; size_of::<TimeSpec32>()];
         self.read_mem(&mut buf, addr)?;
 
         // SAFETY: The following unsafe block assumes that:
@@ -1186,7 +1188,7 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; std::mem::size_of::<TimeSpec64>()];
+        let mut buf = [0u8; size_of::<TimeSpec64>()];
         self.read_mem(&mut buf, addr)?;
 
         // SAFETY: The following unsafe block assumes that:
@@ -1220,7 +1222,7 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; std::mem::size_of::<TimeSpec32>() * 2];
+        let mut buf = [0u8; size_of::<TimeSpec32>() * 2];
         self.read_mem(&mut buf, addr)?;
 
         // SAFETY: The following unsafe block assumes that:
@@ -1262,7 +1264,7 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; std::mem::size_of::<TimeSpec64>() * 2];
+        let mut buf = [0u8; size_of::<TimeSpec64>() * 2];
         self.read_mem(&mut buf, addr)?;
 
         // SAFETY: The following unsafe block assumes that:
