@@ -109,7 +109,7 @@ macro_rules! impl_rand_uint {
             #[inline]
             fn rand_from_os() -> Result<Self, Errno> {
                 // Read exactly size_of::<$t>() bytes, and interpret in native endian.
-                let mut buf = [0u8; { std::mem::size_of::<$t>() }];
+                let mut buf = [0u8; { size_of::<$t>() }];
                 fillrandom(&mut buf)?;
                 Ok(<$t>::from_ne_bytes(buf))
             }
