@@ -30,6 +30,7 @@ use nix::{
         resource::{setrlimit, Resource, RLIM_INFINITY},
         utsname::uname,
     },
+    unistd::isatty,
 };
 use syd::config::*;
 
@@ -49,6 +50,20 @@ impl<E: Error> From<E> for TestError {
     }
 }
 
+#[macro_export]
+macro_rules! say {
+    ($($arg:tt)*) => {{
+        let msg = format!($($arg)*);
+        let pre = "\r\x1b[K";
+
+        if *$crate::util::SYD_TEST_TTY {
+            eprint!("{pre}{msg}");
+        } else {
+            eprintln!("{msg}");
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! assert {
     ($cond:expr) => {
@@ -156,7 +171,7 @@ macro_rules! assert_status_panicked {
 macro_rules! assert_status_code_matches {
     ($status:expr, $($pattern:tt)+) => {{
         let code = $status.code().unwrap_or(127);
-        assert!(
+        $crate::assert!(
             matches!(code, $($pattern)+),
             "code: {code} status: {:?}",
             $status
@@ -168,7 +183,7 @@ macro_rules! assert_status_code_matches {
 macro_rules! assert_status_code {
     ($status:expr, $expected:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert_eq!(code, $expected, "code:{code} status:{:?}", $status);
+        $crate::assert_eq!(code, $expected, "code:{code} status:{:?}", $status);
     };
 }
 
@@ -176,7 +191,7 @@ macro_rules! assert_status_code {
 macro_rules! assert_status_ok {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!($status.success(), "code:{code} status:{:?}", $status);
+        $crate::assert!($status.success(), "code:{code} status:{:?}", $status);
     };
 }
 
@@ -184,56 +199,77 @@ macro_rules! assert_status_ok {
 macro_rules! assert_status_not_ok {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!(!$status.success(), "code:{code} status:{:?}", $status);
+        $crate::assert!(!$status.success(), "code:{code} status:{:?}", $status);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_bad_file {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EBADF);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_bad_message {
     ($status:expr) => {
-        assert_status_code!($status, libc::EBADMSG);
+        $crate::assert_status_code!($status, libc::EBADMSG);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_access_denied {
     ($status:expr) => {
-        assert_status_code!($status, libc::EACCES);
+        $crate::assert_status_code!($status, libc::EACCES);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_permission_denied {
     ($status:expr) => {
-        assert_status_code!($status, libc::EPERM);
+        $crate::assert_status_code!($status, libc::EPERM);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_broken_pipe {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EPIPE);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_hidden {
     ($status:expr) => {
-        assert_status_code!($status, libc::ENOENT);
+        $crate::assert_status_code!($status, libc::ENOENT);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_invalid {
     ($status:expr) => {
-        assert_status_code!($status, libc::EINVAL);
+        $crate::assert_status_code!($status, libc::EINVAL);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_interrupted {
     ($status:expr) => {
-        assert_status_code!($status, libc::EINTR);
+        $crate::assert_status_code!($status, libc::EINTR);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_status_loop {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::ELOOP);
     };
 }
 
 #[macro_export]
 macro_rules! assert_status_unimplemented {
     ($status:expr) => {
-        assert_status_code!($status, libc::ENOSYS);
+        $crate::assert_status_code!($status, libc::ENOSYS);
     };
 }
 
@@ -241,7 +277,7 @@ macro_rules! assert_status_unimplemented {
 macro_rules! assert_status_killed {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert_eq!(
+        $crate::assert_eq!(
             code,
             128 + libc::SIGKILL,
             "code:{code} status:{:?}",
@@ -254,7 +290,7 @@ macro_rules! assert_status_killed {
 macro_rules! assert_status_sigsys {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert_eq!(code, 128 + libc::SIGSYS, "code:{code} status:{:?}", $status);
+        $crate::assert_eq!(code, 128 + libc::SIGSYS, "code:{code} status:{:?}", $status);
     };
 }
 
@@ -262,7 +298,7 @@ macro_rules! assert_status_sigsys {
 macro_rules! assert_status_signaled {
     ($status:expr, $signal:expr) => {
         let signal = $status.signal().unwrap_or(127);
-        assert_eq!(signal, $signal, "signal:{signal} status:{:?}", $status);
+        $crate::assert_eq!(signal, $signal, "signal:{signal} status:{:?}", $status);
     };
 }
 
@@ -270,7 +306,7 @@ macro_rules! assert_status_signaled {
 macro_rules! assert_status_not_supported {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!(
+        $crate::assert!(
             matches!(code, libc::EAFNOSUPPORT | nix::libc::ENOTSUP),
             "code:{code} status:{:?}",
             $status
@@ -282,7 +318,7 @@ macro_rules! assert_status_not_supported {
 macro_rules! assert_status_operation_not_supported {
     ($status:expr) => {
         let code = $status.code().unwrap_or(127);
-        assert!(
+        $crate::assert!(
             matches!(code, libc::EOPNOTSUPP),
             "code:{code} status:{:?}",
             $status
@@ -560,6 +596,13 @@ macro_rules! skip_unless_unshare {
             panic!("No namespace given!");
         }
 
+        // Gitlab builds run under docker, and do not permit subnamespaces.
+        if *GL_BUILD {
+            eprintln!("Test requires Linux namespaces ({:?}), skipping!", [$($ns),*]);
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+
         match syd::confine::ns_enabled(flags) {
             Ok(false) => {
                 eprintln!("Test requires Linux namespaces ({:?}), skipping!", [$($ns),*]);
@@ -829,6 +872,11 @@ pub static SYD_PAUSE: LazyLock<String> =
 pub static SYD_PDS: LazyLock<String> =
     LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-pds").unwrap_or("syd-pds".to_string()));
 
+/// Holds a `String` to run `syd-sec`.
+/// Honours CARGO_BIN_EXE_syd-sec environment variable.
+pub static SYD_SEC: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-sec").unwrap_or("syd-sec".to_string()));
+
 /// Holds a `String` to run `syd-size`.
 /// Honours CARGO_BIN_EXE_syd-size environment variable.
 pub static SYD_SIZE: LazyLock<String> =
@@ -874,6 +922,10 @@ pub static CI_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("JOB_ID").ok().i
 /// A boolean which specifies if we're running under GitLab CI.
 pub static GL_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("CI_PROJECT_ID").ok().is_some());
 
+/// A boolean which specifies whether standard error is a TTY.
+pub static SYD_TEST_TTY: LazyLock<bool> =
+    LazyLock::new(|| isatty(std::io::stderr()).unwrap_or(false));
+
 pub struct Syd {
     cmd: Command,
     is_quiet: bool,
@@ -1113,8 +1165,8 @@ pub fn syd() -> Syd {
         cmd.env(ENV_QUIET_TTY, "YesPlease");
     }
     // Allow coredumps and non-PIE.
+    cmd.m("trace/allow_unsafe_exec_nopie:1"); // Allow non-PIE execution.
     cmd.m("trace/allow_unsafe_prlimit:1"); // Allow coredumps.
-    cmd.m("trace/allow_unsafe_nopie:1"); // Allow non-PIE.
     if *USE_TRACE || (*USE_STRACE && env::var_os("SYD_TEST_STRACE_NOFORK").is_none()) {
         cmd.m("trace/allow_unsafe_ptrace:1"); // Disable ptrace.
     }
