@@ -22,8 +22,8 @@ use crate::{
     caps::Capability,
     config::*,
     confine::{confine_scmp_ioctl, confine_scmp_pwritev2, safe_drop_cap},
-    fs::retry_on_eintr,
     landlock::{CompatLevel, Compatible, RestrictSelfFlags, Ruleset, RulesetAttr, Scope},
+    retry::retry_on_eintr,
     unshare::{
         error::ErrorCode as Err,
         run::{ChildInfo, Exe},
@@ -111,14 +111,12 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
     // Apply a Landlock scope sandbox to restrict
     // 1. Ptrace attach outside Landlock.
     // 2. Signal send outside Landlock.
-    // 3. UNIX abstract socket connect outside Landlock.
     // We leave path and network restrictions for Landlock
     // to be configured by the user using Lock sandboxing.
     // This is an added layer for hardening and it's best-effort.
     let mut ruleset = Ruleset::default();
     let ruleset_ref = &mut ruleset;
     ruleset_ref.set_compatibility(CompatLevel::BestEffort);
-    let _ = ruleset_ref.scope(Scope::AbstractUnixSocket);
     let _ = ruleset_ref.scope(Scope::Signal);
     if let Ok(ruleset) = ruleset.create() {
         let _ = ruleset.restrict_self(RestrictSelfFlags::empty());
