@@ -16,7 +16,6 @@ use syd::{
     err::SydResult,
     landlock_policy::LandlockPolicy,
     path::XPathBuf,
-    syslog::LogLevel,
 };
 
 // Set global allocator to mimalloc.
@@ -34,12 +33,6 @@ syd::main! {
 
     syd::set_sigpipe_dfl()?;
 
-    // Initialize logging.
-    syd::log::log_init_simple(LogLevel::Warn)?;
-
-    // Configure syd::proc.
-    syd::config::proc_init()?;
-
     // Parse CLI options.
     let mut etyp = false;
     let mut is_32bit = false;
