@@ -9,10 +9,9 @@
 use std::process::ExitCode;
 
 use nix::{errno::Errno, libc::pid_t, unistd::Pid};
-use procfs_core::process::MMapPath;
 use syd::{
     human_size,
-    proc::{proc_smaps, proc_statm},
+    proc::{proc_mem, proc_smaps, proc_statm},
 };
 
 // Set global allocator to mimalloc.
@@ -31,9 +30,10 @@ syd::main! {
     syd::set_sigpipe_dfl()?;
 
     // Configure syd::proc.
-    syd::config::proc_init()?;
+    syd::config::proc_init_simple()?;
 
     // Parse CLI options.
+    let mut opt_smaps = false; // -s (use the slow mode).
     let mut opt_human = false; // -H
     let mut opt_is_vm = false; // -V
     let mut opt_pid = None;
@@ -45,12 +45,9 @@ syd::main! {
                 help();
                 return Ok(ExitCode::SUCCESS);
             }
-            Short('H') => {
-                opt_human = true;
-            }
-            Short('V') => {
-                opt_is_vm = true;
-            }
+            Short('H') => opt_human = true,
+            Short('V') => opt_is_vm = true,
+            Short('s') => opt_smaps = true,
             Value(pid) if opt_pid.is_none() => {
                 opt_pid = Some(pid.parse::<pid_t>()?);
             }
@@ -59,20 +56,28 @@ syd::main! {
     }
 
     let pid = match opt_pid {
-        None => Pid::parent().as_raw(),
-        Some(pid) => pid,
+        None => Pid::parent(),
+        Some(pid) => Pid::from_raw(pid),
     };
 
     let size = if opt_is_vm {
-        match proc_statm(Pid::from_raw(pid)) {
-            Ok(statm) => (statm.size as u64).saturating_mul(*syd::config::PAGE_SIZE),
+        match proc_statm(pid) {
+            Ok(statm) => statm.size.saturating_mul(*syd::config::PAGE_SIZE),
+            Err(error) => {
+                eprintln!("syd-mem: {error}");
+                return Ok(ExitCode::FAILURE);
+            }
+        }
+    } else if opt_smaps {
+        match proc_mem_smaps(pid) {
+            Ok(size) => size,
             Err(error) => {
                 eprintln!("syd-mem: {error}");
                 return Ok(ExitCode::FAILURE);
             }
         }
     } else {
-        match proc_mem(Pid::from_raw(pid)) {
+        match proc_mem(pid) {
             Ok(size) => size,
             Err(error) => {
                 eprintln!("syd-mem: {error}");
@@ -97,55 +102,20 @@ fn help() {
     println!("-V    Print virtual memory size");
 }
 
-/// Calculates process memory usage.
+/// Calculates process memory usage (slowly) using proc_pid_smaps(5).
 ///
-/// This function uses the `procfs` crate to obtain detailed memory maps
-/// from `/proc/[pid]/smaps`. It sums multiple memory usage values reported in these maps
-/// to calculate a more comprehensive total memory usage.
-///
-/// # Arguments
-///
-/// * `process` - `Process` instance representing the process.
-///
-/// # Returns
-///
-/// This function returns a `Result<u64, Errno>`.
-///
-/// # Errors
-///
-/// This function returns an error if it fails to retrieve the process's memory maps,
-/// typically due to insufficient permissions or an invalid process ID.
-fn proc_mem(pid: Pid) -> Result<u64, Errno> {
+/// This function iterates through proc_pid_smaps(5), and therefore it's less efficient than
+/// `syd::proc::proc_mem` which calculates memory usage using the file _proc_pid_smaps_rollup(5)
+/// in one go. See: https://www.kernel.org/doc/Documentation/ABI/testing/procfs-smaps_rollup
+fn proc_mem_smaps(pid: Pid) -> Result<u64, Errno> {
     proc_smaps(pid).map(|maps| {
-        let mut total_size: u64 = 0;
+        let mut sum = 0u64;
         for map in &maps {
-            match &map.0.pathname {
-                MMapPath::Path(_) | MMapPath::Anonymous | MMapPath::Stack | MMapPath::Other(_) => {
-                    let pss = map.0.extension.map.get("Pss").copied().unwrap_or(0);
-                    let private_dirty = map
-                        .0
-                        .extension
-                        .map
-                        .get("Private_Dirty")
-                        .copied()
-                        .unwrap_or(0);
-                    let shared_dirty = map
-                        .0
-                        .extension
-                        .map
-                        .get("Shared_Dirty")
-                        .copied()
-                        .unwrap_or(0);
-
-                    total_size = total_size.saturating_add(
-                        pss.saturating_add(private_dirty)
-                            .saturating_add(shared_dirty),
-                    );
-                }
-                _ => (),
+            for key in ["Pss", "Private_Dirty", "Shared_Dirty"] {
+                let val = map.0.extension.map.get(key).copied().unwrap_or(0);
+                sum = sum.saturating_add(val);
             }
         }
-
-        total_size
+        sum
     })
 }
