@@ -39,8 +39,10 @@ use nix::{
 use syd::{
     config::{ENV_SH, SYD_SH},
     confine::run_cmd,
-    fs::{lock_fd, retry_on_eintr, set_cloexec},
+    fs::set_cloexec,
+    ofd::lock_fd,
     path::XPathBuf,
+    retry::retry_on_eintr,
     timer::AlarmTimer,
 };
 
