@@ -80,6 +80,7 @@ syd::main! {
                     .or(Err(Errno::EINVAL))?
             }
             Short('B') => fsflags.insert(FsFlags::RESOLVE_BENEATH),
+            Short('R') => fsflags.insert(FsFlags::RESOLVE_IN_ROOT),
             Short('D') => fsflags.insert(FsFlags::NO_RESOLVE_DOTDOT),
             Short('F') => fsflags.insert(FsFlags::NO_RESOLVE_PATH),
             Short('M') => fsflags.insert(FsFlags::MISS_LAST),
@@ -109,6 +110,12 @@ syd::main! {
         return Err(Errno::EINVAL.into());
     }
 
+    // -B conflicts with -R:
+    if fsflags.contains(FsFlags::RESOLVE_BENEATH | FsFlags::RESOLVE_IN_ROOT) {
+        eprintln!("syd-read: -B and -R options are mutually exclusive!");
+        return Err(Errno::EINVAL.into());
+    }
+
     // Open beneath directory if given.
     #[expect(clippy::disallowed_methods)]
     let opt_dir = if let Some(ref dir) = opt_dir {
@@ -168,7 +175,7 @@ syd::main! {
 }
 
 fn help() {
-    println!("Usage: syd-read [-hmnzBDFMNPUX] [-c n] [-d dir] [-p pid] path...");
+    println!("Usage: syd-read [-hmnzBDFMNPRUX] [-c n] [-d dir] [-p pid] path...");
     println!("Print resolved symbolic links or canonical file names.");
     println!("By default last component may exist, other components must exist.");
     println!("  -h       Print this help message and exit.");
@@ -179,7 +186,8 @@ fn help() {
     println!(
         "  -M       Last component must not exist, other components must exist, conflicts with -m."
     );
-    println!("  -B       Resolve beneath the given directory, useful with -d <dir>.");
+    println!("  -B       Resolve beneath the given directory, useful with -d <dir>. Implies -P, conflicts with -R");
+    println!("  -R       Treat the directory as root directory, useful with -d <dir>. Implies -P, conflicts with -B");
     println!("  -D       Do not traverse through `..` components.");
     println!("  -X       Do not traverse through mount points.");
     println!("  -F       Do not follow symbolic links for any of the path components.");
