@@ -19,11 +19,14 @@ use nix::{
         stat::lstat,
         wait::{waitpid, WaitPidFlag, WaitStatus},
     },
-    unistd::{fork, ForkResult},
+    unistd::{fork, ForkResult, Pid},
 };
 use syd::{
+    config::proc_init_simple,
     confine::print_seccomp_architectures,
+    fs::unix_inodes,
     ioctl::{ioctl_map_get, ioctl_names_get, Ioctl},
+    proc::proc_unix_inodes,
     wildmatch::inamematch,
 };
 
@@ -50,6 +53,8 @@ syd::main! {
     let mut opt_probe = false; // -p
     let mut opt_open = false; // -o
     let mut opt_signal = false; // -s
+    let mut opt_unix_nl = false; // -u: netlink(7)
+    let mut opt_unix_pn = false; // -U: proc_net(5)
     let mut opt_tmout = Duration::from_secs(3); // -t
     let mut opt_sys = None;
     let mut opt_arg = Vec::new();
@@ -66,6 +71,8 @@ syd::main! {
             Short('p') => opt_probe = true,
             Short('o') => opt_open = true,
             Short('s') => opt_signal = true,
+            Short('u') => opt_unix_nl = true,
+            Short('U') => opt_unix_pn = true,
             Short('g') => {
                 // -g doesn't make sense without -p.
                 opt_ghost = true;
@@ -103,12 +110,37 @@ syd::main! {
         }
     }
 
-    let flags = [opt_errno, opt_ioctl, opt_open, opt_signal];
+    let flags = [opt_errno, opt_ioctl, opt_open, opt_signal, opt_unix_nl, opt_unix_pn];
     if flags.iter().filter(|&&flag| flag).count() > 1 {
-        eprintln!("At most one of -e, -i, -o and -s must be given!");
+        eprintln!("sys-sys: At most one of -e, -i, -o, -s, -u, and -U must be given!");
         return Err(Errno::EINVAL.into());
     }
 
+    if opt_unix_nl {
+        if opt_sys.is_some() {
+            eprintln!("syd-sys: -u does not accept a parameter!");
+            return Err(Errno::EINVAL.into());
+        }
+
+        for inode in unix_inodes()? {
+            println!("{inode}");
+        }
+
+        return Ok(ExitCode::SUCCESS);
+    } else if opt_unix_pn {
+        if opt_sys.is_some() {
+            eprintln!("syd-sys: -U does not accept a parameter!");
+            return Err(Errno::EINVAL.into());
+        }
+
+        proc_init_simple()?;
+        for inode in proc_unix_inodes(Pid::this())? {
+            println!("{inode}");
+        }
+
+        return Ok(ExitCode::SUCCESS);
+    }
+
     let sysarg = if let Some(value) = opt_sys {
         value
     } else {
@@ -123,7 +155,7 @@ syd::main! {
         } else {
             "syscall"
         };
-        eprintln!("Expected {what} number or name regex as first argument!");
+        eprintln!("syd-sys: Expected {what} number or name regex as first argument!");
         return Ok(ExitCode::FAILURE);
     };
 
@@ -354,7 +386,7 @@ syd::main! {
             args[argc] = match value.parse::<libc::c_long>() {
                 Ok(value) => Some(value),
                 Err(error) => {
-                    eprintln!("Argument {argc} is invalid: {error}");
+                    eprintln!("syd-sys: Argument {argc} is invalid: {error}");
                     return Ok(ExitCode::FAILURE);
                 }
             };
@@ -371,7 +403,7 @@ syd::main! {
 }
 
 fn help() {
-    println!("Usage: syd-sys [-hgeiopst] [-a list|native|x86|x86_64|aarch64...] number|name-glob [<probe-args>...]");
+    println!("Usage: syd-sys [-hgeiopstuU] [-a list|native|x86|x86_64|aarch64...] number|name-glob [<probe-args>...]");
     println!("Given a number, print the matching syscall name and exit.");
     println!("Given a glob, print case-insensitively matching syscall names and exit.");
     println!("Given -e, query errnos rather than syscalls.");
@@ -381,6 +413,8 @@ fn help() {
     println!("Given -p, probe the system call and print result.");
     println!("Given -g with -p, enable Syd's Ghost mode prior to probing.");
     println!("Use -t to specify syscall probe timeout in seconds, defaults to 3 seconds.");
+    println!("Use -u to list UNIX domain socket inodes using netlink(7)");
+    println!("Use -U to list UNIX domain socket inodes using proc_net(5)");
 }
 
 fn probe_syscall(
