@@ -196,8 +196,8 @@ syd::main! {
         fillrandom(addr_vec)?;
 
         // Find a random aligned offset within the usable range.
-        let addr_size = std::mem::size_of::<SockaddrStorage>();
-        let addr_align = std::mem::align_of::<SockaddrStorage>();
+        let addr_size = size_of::<SockaddrStorage>();
+        let addr_align = align_of::<SockaddrStorage>();
         let max_offset = page_siz.get().checked_sub(addr_size).ok_or(Errno::EINVAL)?;
         let rand_off = randint(0..=max_offset as u64)? as usize;
         let aligned_off = rand_off & !(addr_align - 1);
@@ -1227,7 +1227,7 @@ fn set_socket_options<F: AsFd>(fd: &F, backlog: Option<libc::c_int>) {
                 level,
                 optname,
                 &optval as *const _ as *const libc::c_void,
-                std::mem::size_of::<T>() as libc::socklen_t,
+                size_of::<T>() as libc::socklen_t,
             )
         };
         if ret == -1 {
