@@ -40,14 +40,14 @@ use nix::{
 };
 
 use crate::{
-    compat::{pipe2_raw, waitid, WaitStatus},
+    compat::{pipe2_raw, waitid, MFdFlags, WaitStatus},
     config::{MINI_STACK_SIZE, *},
     confine::{confine_mdwe, CLONE_NEWTIME, MS_NOSYMFOLLOW},
+    cookie::safe_memfd_create,
     debug,
     err::err2no,
     fs::{
-        create_memfd, pidfd_send_signal, safe_clone, safe_copy_if_exists, seal_memfd, set_cloexec,
-        set_nonblock, MFD_ALLOW_SEALING,
+        pidfd_send_signal, safe_clone, safe_copy_if_exists, seal_memfd, set_cloexec, set_nonblock,
     },
     get_user_home, get_user_name,
     hash::SydHashSet,
@@ -307,7 +307,11 @@ impl WordExp<'_> {
         // Create a memory fd to write input into,
         // and pass to the internal /bin/sh invoked
         // by wordexp(3).
-        let mut file = create_memfd(b"syd-wordexp\0", MFD_ALLOW_SEALING).map(File::from)?;
+        let mut file = safe_memfd_create(
+            c"syd-wordexp",
+            MFdFlags::MFD_ALLOW_SEALING | MFdFlags::MFD_CLOEXEC,
+        )
+        .map(File::from)?;
         debug!("ctx": "expand",
             "msg": format!("created memory-file {} with close-on-exec flag set",
                 file.as_raw_fd()));
@@ -317,7 +321,7 @@ impl WordExp<'_> {
         file.write_all(b"\n")?;
 
         // Handle system-wide configuration.
-        safe_copy_if_exists(&mut file, XPath::from_bytes(b"/etc/syd/init.sh"))?;
+        safe_copy_if_exists(&mut file, "/etc/syd/init.sh")?;
         file.write_all(b"\n")?;
 
         // Handle user-specific configuration.
@@ -1016,7 +1020,7 @@ const WORDEXP_SYSCALLS: &[&str] = &[
     "llistxattr",
     "lseek",
     "lstat",
-    "madvise", // TODO: confine advices (no-op!).
+    "madvise", // TODO: confine advice (no-op!).
     "membarrier",
     "mlock",
     "mlock2",
