@@ -35,8 +35,8 @@ use crate::cookie::FTRUNCATE64_COOKIE_ARG3;
 use crate::{
     config::*,
     confine::{
-        confine_scmp_clone, confine_scmp_clone3, confine_scmp_fadvise, confine_scmp_write,
-        scmp_add_setid_rules, ExportMode,
+        confine_scmp_clone, confine_scmp_clone3, confine_scmp_fadvise, confine_scmp_madvise,
+        confine_scmp_write, scmp_add_setid_rules, ExportMode,
     },
     cookie::{
         safe_ftruncate64, safe_pipe2, ACCEPT4_COOKIE_ARG4, ACCEPT4_COOKIE_ARG5,
@@ -47,13 +47,15 @@ use crate::{
     debug,
     err::{err2no, SydJoinHandle, SydResult},
     error,
-    fs::{lock_fd, retry_on_eintr, retry_on_intr, seal_memfd, FileInfo},
+    fs::{seal_memfd, FileInfo},
     hash::{
         aes_ctr_enc, aes_ctr_init, hmac_sha256_feed, hmac_sha256_fini, hmac_sha256_init,
         SydHashMap, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE, SYD3_HDR_SIZE,
     },
     info,
+    ofd::lock_fd,
     path::{XPath, XPathBuf},
+    retry::{retry_on_eintr, retry_on_intr},
     sandbox::Flags,
 };
 
@@ -191,6 +193,9 @@ impl AesWorker {
         // We need this because std::thread::spawn has unwanted
         // side-effects such as opening /sys/devices/system/cpu/online
         // on some architectures.
+        //
+        // Note, we avoid this when profiling is enabled,
+        // as gperf requires it to write profiling data.
         for sysname in [
             "open",
             "openat",
@@ -202,7 +207,12 @@ impl AesWorker {
         ] {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Errno(Errno::ENOSYS as i32), syscall)?;
+                    let action = if !cfg!(feature = "prof") {
+                        ScmpAction::Errno(Errno::ENOSYS as i32)
+                    } else {
+                        ScmpAction::Allow
+                    };
+                    ctx.add_rule(action, syscall)?;
                 }
                 Err(_) => {
                     info!("ctx": "confine", "op": "allow_aes_syscall",
@@ -248,6 +258,9 @@ impl AesWorker {
         // Allow fadvise family syscalls.
         confine_scmp_fadvise(&mut ctx)?;
 
+        // Allow safe madvise(2) advice.
+        confine_scmp_madvise(&mut ctx)?;
+
         // ftruncate{,64}(2) may be used only with syscall argument cookies.
         let sysname = "ftruncate";
         #[expect(clippy::useless_conversion)]
@@ -868,7 +881,7 @@ impl AesWorker {
         let last_block_offset = options.last_block_offset;
         debug!("ctx": "aes", "op": "authenticate_text",
             "path": crypt_path, "memfd": memfd, "mode": file_mode,
-            "msg": "feed ciphertext into HMAC algoritm socket");
+            "msg": "feed ciphertext into HMAC algorithm socket");
 
         #[expect(clippy::cast_possible_wrap)]
         lseek64(&enc_fd, SYD3_HDR_SIZE as i64, Whence::SeekSet)?;
