@@ -302,7 +302,7 @@ impl Interrupter {
                 .sig_restart
                 .lock()
                 .unwrap_or_else(|err| err.into_inner())
-                .get(&Pid::from_raw(status.pid))
+                .get(&status.pid)
             {
                 sigset.del_set(*sigset_restart);
 
@@ -455,10 +455,18 @@ impl Interrupter {
         // We need this because std::thread::spawn has unwanted
         // side-effects such as opening /sys/devices/system/cpu/online
         // on some architectures.
+        //
+        // Note, we avoid this when profiling is enabled,
+        // as gperf requires it to write profiling data.
         for sysname in ["open", "openat", "stat", "lstat", "statx", "newfstatat"] {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Errno(Errno::ENOSYS as i32), syscall)?;
+                    let action = if !cfg!(feature = "prof") {
+                        ScmpAction::Errno(Errno::ENOSYS as i32)
+                    } else {
+                        ScmpAction::Allow
+                    };
+                    ctx.add_rule(action, syscall)?;
                 }
                 Err(_) => {
                     info!("ctx": "confine", "op": "allow_int_syscall",
