@@ -322,11 +322,10 @@ impl<'a> WorkerCache<'a> {
 // maximum value that can be stored within half the bits of usize, as
 // two counters (total workers and busy workers) are stored in one
 // AtomicUsize.
-const BITS: usize = std::mem::size_of::<usize>() * 8;
-const MAX_SIZE: usize = (1 << (BITS / 2)) - 1;
+const MAX_SIZE: usize = (1 << (usize::BITS / 2)) - 1;
 
 const WORKER_BUSY_MASK: usize = MAX_SIZE;
-const INCREMENT_TOTAL: usize = 1 << (BITS / 2);
+const INCREMENT_TOTAL: usize = 1 << (usize::BITS / 2);
 const INCREMENT_BUSY: usize = 1;
 
 /// 1. Struct containing data shared between workers.
@@ -386,14 +385,14 @@ impl WorkerData {
 
     #[inline]
     pub(crate) fn split(val: usize) -> (usize, usize) {
-        let total_count = val >> (BITS / 2);
+        let total_count = val >> (usize::BITS / 2);
         let busy_count = val & WORKER_BUSY_MASK;
         (total_count, busy_count)
     }
 
     #[inline]
     fn total(val: usize) -> usize {
-        val >> (BITS / 2)
+        val >> (usize::BITS / 2)
     }
 
     #[inline]
